#!/bin/bash

# Find and set DESTDIR
if [[ ! -n ${DESTDIR} ]]; then
    export DESTDIR="$1"
    if [[ ! -n ${DESTDIR} ]] || [[ "${DESTDIR}" == "overwrite" ]]; then
       export DESTDIR="/"
#       echo "Warning DESTDIR not set, defaulting to /"
    fi
fi

#echo "DESTDIR: ${DESTDIR}"


check_and_add() {
    TARGET=""
    if ls /etc/rc*.d/S*$1* &>/dev/null; then
        TARGET="$2"
    elif ls /etc/runlevels/*/*$1* &>/dev/null; then
        TARGET="$2"
    fi
    if [ ! -z "$TARGET" ]; then
        echo "Adding $2"
        echo "$2" >> "$3"
    fi
}


gen_system() {
echo "system/initial
system/mountroot
system/mountfs
system/bootmisc
system/clock
system/hostname
system/modules
system/static-modules
system/hdparm
system/keymaps
system/urandom
system/consolefont
system/swap
system/coldplug
net/lo" > ${DESTDIR}/etc/initng/system.runlevel

if [ -x /usr/sbin/readahead ] || [ -x /sbin/readahead ]; then
  if [ -e /etc/readahead ]; then
      echo "Adding readahead, NOTE, if you have /usr mounted on another partition:"
      echo "  cp /usr/sbin/readahead /sbin/readahead"
      echo "system/readahead" >> ${DESTDIR}/etc/initng/system.runlevel
  fi
fi

## Getty:
if [ -x /sbin/agetty ]; then
  echo "daemon/agetty" >> ${DESTDIR}/etc/initng/system.runlevel
elif [ -x /sbin/getty ]; then
  echo "daemon/getty" >> ${DESTDIR}/etc/initng/system.runlevel
else
  echo "Failed to find getty/agetty, you must set that manually in /etc/initng/system.runlevel"
fi
}

gen_default() {
echo "system
net/eth0" > ${DESTDIR}/etc/initng/default.runlevel

check_and_add ssh daemon/sshd ${DESTDIR}/etc/initng/default.runlevel
check_and_add dbus daemon/dbus ${DESTDIR}/etc/initng/default.runlevel
check_and_add gdm daemon/gdm ${DESTDIR}/etc/initng/default.runlevel

if ls /etc/dbus*/event.d/*hal* &>/dev/null; then
    #check what version it is.
    hald --help | grep retain-privileges
    if [[ "$?" == "0" ]]; then
	echo "daemon/hald" >> ${DESTDIR}/etc/initng/default.runlevel
    else
	echo "daemon/hald05" >> ${DESTDIR}/etc/initng/default.runlevel
    fi
fi

if [[ -e /usr/sbin/cron ]]; then
    check_and_add cron daemon/vixie-cron ${DESTDIR}/etc/initng/default.runlevel
elif [[ -e /usr/sbin/dcron ]]; then
    check_and_add cron daemon/dcron ${DESTDIR}/etc/initng/default.runlevel
fi


}

gen_up() {
##Header:
echo "#
#	Started when system is up.
#
service up {" > ${DESTDIR}/etc/initng/up.i

#Getty:
if [ -x /sbin/agetty ]; then
  echo "	need = daemon/agetty/tty1" >> ${DESTDIR}/etc/initng/up.i
elif [ -x /sbin/getty ]; then
  echo "	need = daemon/getty/tty1" >> ${DESTDIR}/etc/initng/up.i
else
  echo "Failed to find getty/agetty, you must set that manually in /etc/initng/up.i"
fi

#End:
echo "}" >> ${DESTDIR}/etc/initng/up.i
}

echo "Automatically generating system,runlevel,default.runlevel and up.i"
if [ -e ${DESTDIR}/etc/initng/system.runlevel ] && [ -e ${DESTDIR}/etc/initng/default.runlevel ] && [ -e ${DESTDIR}/etc/initng/up.i ] && [ "$1" != "overwrite" ]; then
  echo "Runlevels already exists, force overwrite with ./gen_system_runlevel.sh overwrite"
  exit 0
fi


if [ -e ${DESTDIR}/etc/initng/system.runlevel ] && [ "$1" != "overwrite" ]; then
  echo "system.runlevel already exists, skipping..."
else
  gen_system
fi

if [ -e ${DESTDIR}/etc/initng/default.runlevel ] && [ "$1" != "overwrite" ]; then
  echo "default.runlevel already exists, skipping..."
else
  gen_default
fi

if [ -e ${DESTDIR}/etc/initng/up.i ] && [ "$1" != "overwrite" ]; then
  echo "up.i already exists, skipping..."
else
  gen_up
fi

echo "Done generating files."
