service system/mountfs {
	need = system/initial system/mountroot
	# system/modules
start {
	# Start software raid.
	startSRaid() 
	{
		# You need a properly configured /etc/raidtab for raidtools usage or a
		# properly configured /etc/mdadm.conf for mdadm usage. Devices in
		# /etc/mdadm.conf are initialized first, so any duplicate devices in
		# /etc/raidtab will not get initialized.
		#if [ -z ${CDBOOT} ] && [ -f /proc/mdstat ] ; then
		if [ -f /proc/mdstat ] ; then
			mdadm_devices=
			raidtools_devices=
			# If /etc/mdadm.conf exists, grab all the RAID devices from it
			if [ -f /etc/mdadm.conf ] ; then
				mdadm_devices=$(awk '/^[[:space:]]*ARRAY/ { print $2 }' /etc/mdadm.conf)
			fi
			# If /etc/raidtab exists, grab all the RAID devices from it
			if [ -f /etc/raidtab ] ; then
				raidtools_devices=$(awk '/^[[:space:]]*raiddev/ { print $2 }' /etc/raidtab)
			fi
			echo "Starting up RAID devices ... "
			rc=0
			retval=0
			for i in ${mdadm_devices} ; do
			raiddev=${i##*/}
				raidstat=$(egrep "^${raiddev} : active" /proc/mdstat)
				if [ -z ${raidstat} ] ; then
					# First scan the /etc/fstab for the "noauto"-flag
					# for this device. If found, skip the initialization
					# for it to avoid dropping to a shell on errors.
					# If not, try raidstart...if that fails then
					# fall back to raidadd, raidrun.  If that
					# also fails, then we drop to a shell
					retval=1
					noauto=$(egrep "^${i}" /etc/fstab | grep -c 'noauto')
						echo "  Trying ${raiddev}..."
						raiddev=""
					
					if [ ${noauto} -gt 0 ] ; then
						retval=0
						raiddev=" (skipped)"
					fi
					if [ ${retval} -gt 0 ] && [ -x /sbin/mdadm ] ; then
						/sbin/mdadm -As "${i}" >/dev/null 2>&1
						retval=$?
					fi
						echo "${raiddev}"
					
					if [ ${retval} -gt 0 ] ; then
						rc=1
						echo "Raid is up..."
					else
						echo "Problems initiating raid..."
					fi
				fi
			done
				for i in ${raidtools_devices} ; do
				raiddev=${i##*/}
				raidstat=$(egrep "^${raiddev} : active" /proc/mdstat)
				if [ -z ${raidstat} ] ; then
					# First scan the /etc/fstab for the "noauto"-flag
					# for this device. If found, skip the initialization
					# for it to avoid dropping to a shell on errors.
					# If not, try raidstart...if that fails then
					# fall back to raidadd, raidrun.  If that
					# also fails, then we drop to a shell
					retval=1
					noauto=$(egrep "^${i}" /etc/fstab | grep -c 'noauto')
						echo "  Trying ${raiddev}..."
						raiddev=""
					if [ ${noauto} -gt 0 ] ; then
						retval=0
						raiddev=" (skipped)"
					fi
					if [ ${retval} -gt 0 ] && [ -x /sbin/raidstart ] ; then
						/sbin/raidstart "${i}"
						retval=$?
					fi
					if [ ${retval} -gt 0 ] && [ -x /sbin/raid0run ] ; then
						/sbin/raid0run "${i}"
						retval=$?
					fi
					if [ ${retval} -gt 0 ] && [ -x /sbin/raidadd ] && [ -x /sbin/raidrun ] ; then
						/sbin/raidadd "${i}"
						/sbin/raidrun "${i}"
						retval=$?
					fi
						echo "${raiddev}"
					if [ ${retval} -gt 0 ] ; then
						rc=1
						echo "Raid is up ..."
						#echo ${retval}
					else
						echo "Problems with raid ..."
						#echo ${retval}
					fi
				fi
			done
			# A non-zero return means there were problems.
			if [ ${rc} -gt 0 ] ; then
				echo
				echo "An error occurred during the RAID startup"
				echo "Dropping you to a shell; the system will reboot"
				echo "when you leave the shell."
				echo; echo
				ngc -j sulogin
				exit 1
				#echo "Unmounting filesystems"
				#/bin/mount -a -o remount,ro >/dev/null 2>&1
				#echo "Rebooting"
				#/sbin/reboot -f
			fi
		fi
	}
	
	#Check all filesystems
	checkfs() 
	{
		if [ -f /fastboot ]
		then
			rm -f /fastboot
		else
			echo "Checking all filesystems"
			if [ -f /forcefsck ] ; then
				echo "A full fsck has been forced"
				/sbin/fsck -C -R -A -a -f
				retval=$?
				rm -f /forcefsck
			else
				/sbin/fsck -C -T -R -A -a
				retval=$?
			fi
			
			if [ ${retval} -eq 0 ] ; then
				echo "All filesystems passed."
			elif [ ${retval} -eq 8 ] ; then
				echo "Filesystem already mounted, skipping."
			elif [ ${retval} -ge 1 ] && [ ${retval} -le 3 ] ; then
				echo "Filesystem errors corrected."
				# Everything should be ok, so return a pass
			else
				echo "Fsck could not correct all errors, manual repair needed"
				ngc -j sulogin
				exit 1
			fi
		fi
	}
	
	# Mount local filesystems in /etc/fstab.
	localmount() 
	{
		echo "Mounting local filesystems ..."
		mount -aFt reiserfs,reiser4,ext2,ext3,xfs,jfs,vfat,ntfs,tmpfs,bind
		if [ ${?} -ne 0 ]; then
		    echo "Mounting failed, trying mount local filesystems again ..."
		    if ! mount -at reiserfs,reiser4,ext2,ext3,xfs,jfs,vfat,ntfs,tmpfs,bind; then
		        echo "WARNING, failed to mount some local filesystems"
		        #this is not critical, lets continue anyway.
		    fi
		fi

		# Mount all type bind.
		mount -at bind	
	}
	
	cleanup()
	{
		echo "Cleaning /var/lock, /var/run ..."
		rm -rf /var/run/console.lock /var/run/console/* &
		#
		# Clean up any stale locks.
		#
		find /var/lock -type f -print0 | xargs -0 rm -f --
		#
		# Clean up /var/run and create /var/run/utmp so that we can login.
		#
		#no need to see if daemons are running, no daemons should access /var/run before checkfs has cleaned it.
		find /var/run/ ! -type d ! -name utmp ! -name innd.pid ! -name random-seed -print0 | xargs -0 rm -f --
	}

	export PATH="/bin:/sbin:/usr/bin:/usr/sbin"
	retval=0
	startSRaid
	checkfs
	localmount
	cleanup
}

}
