/* vi: set sw=4 ts=4: */
/*
 * Busybox main internal header file
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Based in part on code from sash, Copyright (c) 1999 by David I. Bell
 * Permission has been granted to redistribute this code under the GPL.
 *
 */

/* Light version, for use with initng + ash */

#define _GNU_SOURCE

#ifndef	__LIBCONFIG_H__
#define	__LIBCONFIG_H__    1

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <regex.h>
#include <termios.h>
#include <stdint.h>

#include <netdb.h>

#ifdef DMALLOC
#include <dmalloc.h>
#endif

#include <features.h>

#ifdef CONFIG_SELINUX
#include <proc_secure.h>
#endif

#ifdef CONFIG_FEATURE_SHADOWPASSWDS
#include "shadow_.h"
#endif
#ifdef CONFIG_FEATURE_SHA1_PASSWORDS
# include "sha1.h"
#endif

/* Convenience macros to test the version of gcc. */
#if defined __GNUC__ && defined __GNUC_MINOR__
# define __GNUC_PREREQ(maj, min) \
        ((__GNUC__ << 16) + __GNUC_MINOR__ >= ((maj) << 16) + (min))
#else
# define __GNUC_PREREQ(maj, min) 0
#endif

/* __restrict is known in EGCS 1.2 and above. */
#if !__GNUC_PREREQ (2,92)
# define __restrict                         /* Ignore */
#endif

/* Some useful definitions */
#define FALSE   ((int) 0)
#define TRUE    ((int) 1)
#define SKIP	((int) 2)

/* for mtab.c */
#define MTAB_GETMOUNTPT '1'
#define MTAB_GETDEVICE  '2'

#define BUF_SIZE        8192
#define EXPAND_ALLOC    1024

/* Macros for min/max.  */
#ifndef MIN
#define	MIN(a,b) (((a)<(b))?(a):(b))
#endif

#ifndef MAX
#define	MAX(a,b) (((a)>(b))?(a):(b))
#endif

#ifndef DMALLOC
extern void *xmalloc(size_t size);
extern void *xrealloc(void *old, size_t size);
extern void *xcalloc(size_t nmemb, size_t size);
#endif
extern char *bb_xstrdup(const char *s);
extern char *bb_xstrndup(const char *s, int n);
extern char *safe_strncpy(char *dst, const char *src, size_t size);

extern ssize_t safe_read(int fd, void *buf, size_t count);
extern ssize_t safe_write(int fd, const void *buf, size_t count);
extern ssize_t bb_full_write(int fd, const void *buf, size_t len);

const char *u_signal_names(const char *str_sig, int *signo, int startnum);


extern void bb_show_usage(void) __attribute__ ((noreturn));
extern void bb_error_msg(const char *s, ...)
    __attribute__ ((format(printf, 1, 2)));
extern void bb_error_msg_and_die(const char *s, ...)
    __attribute__ ((noreturn, format(printf, 1, 2)));
extern void bb_perror_msg(const char *s, ...)
    __attribute__ ((format(printf, 1, 2)));
extern void bb_perror_msg_and_die(const char *s, ...)
    __attribute__ ((noreturn, format(printf, 1, 2)));
extern void bb_vherror_msg(const char *s, va_list p);
extern void bb_herror_msg(const char *s, ...)
    __attribute__ ((format(printf, 1, 2)));
extern void bb_herror_msg_and_die(const char *s, ...)
    __attribute__ ((noreturn, format(printf, 1, 2)));

extern void bb_perror_nomsg_and_die(void) __attribute__ ((noreturn));
extern void bb_perror_nomsg(void);

/* These two are used internally -- you shouldn't need to use them */
extern void bb_verror_msg(const char *s, va_list p)
    __attribute__ ((format(printf, 1, 0)));
extern void bb_vperror_msg(const char *s, va_list p)
    __attribute__ ((format(printf, 1, 0)));


/* messages.c : */
extern const char *const bb_msg_full_version;
extern const char *const bb_msg_memory_exhausted;
extern const char *const bb_msg_invalid_date;
extern const char *const bb_msg_io_error;
extern const char *const bb_msg_write_error;
extern const char *const bb_msg_name_longer_than_foo;
extern const char *const bb_msg_unknown;
extern const char *const bb_msg_can_not_create_raw_socket;
extern const char *const bb_msg_perm_denied_are_you_root;
extern const char *const bb_msg_standard_input;
extern const char *const bb_msg_standard_output;

extern const char *const bb_path_nologin_file;
extern const char *const bb_path_passwd_file;
extern const char *const bb_path_shadow_file;
extern const char *const bb_path_gshadow_file;
extern const char *const bb_path_group_file;
extern const char *const bb_path_securetty_file;
extern const char *const bb_path_motd_file;

/* THIS IS NOT IN BUSYBOX, why do we need this?!? */

/* messages.c */

#define L_full_version 1
#define L_memory_exhausted 1
#define L_invalid_date 1
#define L_io_error 1
#define L_write_error 1
#define L_name_longer_than_foo 1
#define L_unknown 1
#define L_can_not_create_raw_socket 1
#define L_perm_denied_are_you_root 1
#define L_msg_standard_input 1
#define L_msg_standard_output 1
#define L_passwd_file 1
#define L_shadow_file 1
#define L_group_file 1
#define L_gshadow_file 1
#define L_nologin_file 1
#define L_securetty_file 1
#define L_motd_file 1
#define L_shell_file 1





/* xfuncs.c */
#define L_xmalloc 1
#define L_xrealloc 1
#define L_xcalloc 1
#define L_xstrdup 1
#define L_xstrndup 1
#define L_xfopen 1
#define L_xopen 1
#define L_xread 1
#define L_xread_all 1
#define L_xread_char 1
#define L_xferror 1
#define L_xferror_stdout 1
#define L_xfflush_stdout 1
#define L_strlen 1

/* Fake functions: */
extern int bb_parse_mode(const char *s, mode_t * current_mode);

/* END BLOCK */

#define LIBBB_DEFAULT_LOGIN_SHELL      "-/bin/sh"
extern int bb_default_error_retval;

extern const char *const bb_default_login_shell;

/* "/bin/sh" */
#define DEFAULT_SHELL     (bb_default_login_shell+1)
/* "sh" */
#define DEFAULT_SHELL_SHORT_NAME     (bb_default_login_shell+6)

#endif /* __LIBCONFIG_H__ */
