/* Initng, a next generatind sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"


#include <stdio.h>
#include <stdlib.h>                         /* malloc() free() exit() */
#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_common.h"
#include "../../src/initng_toolbox.h"

#include "initng_depend.h"

s_entry USE = { "use", STRINGS, 3 };
s_entry ALSO_START = { "also_start", STRINGS, 10 };
s_entry ALSO_STOP = { "also_stop", STRINGS, 9 };


/* returns TRUE if all use deps are started */
static int check_use(active_h * service)
{
    active_h *tmp_service = NULL;
    char *use = NULL;

    /* Handle USE */
    while ((use = active_db_get_next_string(&USE, service, use)))
    {
        D_("USE : setting NEED %s in %s\n", use, service->name);
        tmp_service = active_db_find_by_name(use);
        if (!tmp_service)
            continue;
        switch (tmp_service->a_status)
        {
            case RUNNING:
            case DONE:
                continue;                   /* this use is met */
            default:
                D_("\"%s\" still waits for use \"%s\" that has status \"%s\".\n", service->name, tmp_service->name, active_db_get_status_string(tmp_service->a_status));
                return (FALSE);
        }
    }
    return (TRUE);
}

void service_state(active_h * service)
{
    char *tmp = NULL;
    active_h *current;

    switch (service->a_status)
    {
        case START_DEP_MET:
            if (check_use(service) == FALSE)
            {
                mark_service(service, WAITING_FOR_START_DEP);
            }
            return;
        case STARTING:
            /* Handle ALSO_START  (these are not dependencies) */
            while ((tmp =
                    active_db_get_next_string(&ALSO_START, service, tmp)))
                start_new_service_named(tmp);
            /* todo, by listening to STOP_MARKED we can try to stop stuff a bit earlier? */
            return;
        case STOPPED:
            /* Handle ALSO_STOP */
            while ((tmp =
                    active_db_get_next_string(&ALSO_STOP, service, tmp)))
                if ((current = active_db_find_by_name(tmp)))
                {
                    D_("service %s also stops %s\n", service->name, tmp);
                    stop_service(current);
                }
            return;
        default:
            return;
    }
}


int module_init(void)
{
    D_("module_init();\n");
    initng_sdt_add(&USE);
    initng_sdt_add(&ALSO_START);
    initng_sdt_add(&ALSO_STOP);
    initng_add_hook(WATCHERS, &service_state);
    return (TRUE);
}

void module_unload(void)
{
    D_("module_unload();\n");
    initng_sdt_del(&USE);
    initng_sdt_del(&ALSO_START);
    initng_sdt_del(&ALSO_STOP);
    initng_del_hook(WATCHERS, &service_state);
}
