
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"

#include <stdio.h>
#include <stdlib.h>
#include <sys/poll.h>
#include <fcntl.h>
#include <string.h>
#include <time.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <errno.h>
#include <sys/socket.h>
#include <sys/un.h>

#include "../../src/initng_global.h"
#include "../../src/process_db.h"
#include "../../src/service_db.h"
#include "../../src/initng_handler.h"
#include "../../src/initng_history.h"
#include "../../src/main.h"
#include "../../src/active_db.h"
#include "../../src/initng_toolbox.h"
#include "../../src/initng_plugin_hook.h"
#include "../../src/initng_load_module.h"
#include "../../src/initng_plugin_callers.h"
#include "../../src/initng_global.h"
#include "../../src/initng_error.h"


#include "initng_ngc2.h"
#include "print_service.h"

static void accepted_client(void);
static void closesock(void);
static void doread(int sock);
static void addsock(void);
static int sendping(void);
static int open_socket(void);
static void check_socket(void);

/*
   In the Linux implementation, sockets which are visible in the filesystem
   honour the permissions of the directory they are in.  Their owner, group
   and their permissions can be changed.  Creation of a new socket will
   fail if the process does not have write and search (execute) permission
   on the directory the socket is created in.  Connecting to the socket
   object requires read/write permission.  This behavior differs from  many
   BSD derived systems which ignore permissions for Unix sockets.  Portable
   programs should not rely on this feature for security.
 */


/* globals */
int serversock;
struct stat sock_stat;
const char *socket_filename;

static void closesock(void)
{
    /* Check if we need to remove hooks */
    if (serversock == 0)
        return;
    D_("closesock %d\n", serversock);
    initng_del_hook(FDWATCHERS, &accepted_client);

    /* close socket and set to 0 */
    close(serversock);
    serversock = 0;
}

static void doread(int sock)
{
    char buf[512];
    char send_buf[2001];
    int len = 0;
    active_h *apt = NULL;       /* a temp active struct */

    D_("Client accepted, reading from socket\n");
    len = read(sock, buf, sizeof(buf));
    if (len < 1)
        return;
    buf[len] = '\0';
    /* ping check : */
    if (buf[0] == 'X')
    {
        D_("Ping received, sending pong\n");
        write(sock, "Y", 1);
        return;
    }
    /* Todo parse data and return data : */
    switch (buf[0])
    {
            /* This should never be done, down plugin should handle this */
            /*        case 'a':                          / unload all plugins /
               {
               write(sock, "1", 1);        / Write before, we can't write after. /
               initng_close_all_fd();
               break;
               } */
        case 'g':                          /* get pid of */
            {
                char pid[101];

                if (buf + 1 &&
                    (apt = active_db_find_by_name(buf + 1)) &&
                    apt && apt->start_process && apt->start_process->pid)
                {
                    sprintf(pid, "%i", apt->start_process->pid);
                    write(sock, pid, strlen(pid));
                }
                else
                {
                    write(sock, "0", 1);
                }
                break;
            }

        case 'u':                          /* start service */
            {
                if (buf + 1 && start_new_service_named(buf + 1))
                {
                    write(sock, "1", 1);
                }
                else
                {
                    write(sock, "0", 1);
                }
                break;
            }

        case 'j':                          /* stop_all then start */
            {
                if (buf + 1)
                {
                    g.when_out = THEN_NEW_RUNLEVEL;
                    g.runlevel = i_strdup(buf + 1);
                    stop_all();
                    write(sock, "1", 1);
                }
                else
                {
                    write(sock, "0", 1);
                }
                break;
            }

        case 'z':                          /* free service */
            {
                if (buf + 1)
                {
                    apt = active_db_find_by_name(buf + 1);
                    active_db_free(apt);
                    write(sock, "1", 1);
                }
                else
                {
                    write(sock, "0", 1);
                }
                break;
            }

        case 'd':                          /* take down service */
            if (buf + 1 &&
                (apt = active_db_find_by_name(buf + 1))
                && (stop_service(apt)))
                write(sock, "1", 1);
            else
                write(sock, "0", 1);
            break;

        case 'r':                          /* restart service */
            if (buf + 1 &&
                (apt = active_db_find_by_name(buf + 1))
                && (restart_service(apt)))
                write(sock, "1", 1);
            else
                write(sock, "0", 1);
            break;

        case 'h':
            {                               /* print history */
                history_h *tmp = NULL;
                int max = 0;

                D_("parse_control_input(): Sending History\n history:");
                while ((tmp = history_get_next(tmp)))
                {
                    if (tmp && tmp->name)
                    {
                        D_(" %s ", tmp->name);
                        sprintf(send_buf, "%i|%li|%s\n", tmp->action,
                                (long int) tmp->time, tmp->name);
                        send_buf[2001] = '\0';  /* to make sure it stops on buffer overflow */
                        write(sock, send_buf, strlen(send_buf));
                    }
                    if (max++ >= (HISTORY - 1))
                        break;
                }
            }
            break;

        case 's':
            {                               /* print active list */

                D_("parse_control_input(): Sending Running process list\n sending :");
                while ((apt = active_db_next(apt)))
                {
                    if (apt && apt->name)
                    {
                        D_(" %s->%i ", apt->name, apt->a_status);
                        /*printf("%s|<time>|%s\n", active_db_get_status_string(apt->a_status), apt->name); */
                        sprintf(send_buf, "%i|%li|%s\n", apt->a_status,
                                (long int) apt->time_got_status, apt->name);
                        write(sock, send_buf, strlen(send_buf));
                    }
                }
            }
            break;

        case 't':                          /* print uptime */
            if (buf + 1 && (apt = active_db_find_by_name(buf + 1)))
            {
                char tmp_time[10];

                sprintf(tmp_time, "%i",
                        (int) difftime(time(0), apt->time_got_status));
                write(sock, tmp_time, strlen(tmp_time));
            }
            break;

        case 'R':
            {                               /* reload service data, actually removes service data, and it will be loaded when needed. */
                service_h *s;

                if (buf + 1 && (s = service_db_find_by_name(buf + 1)))
                {
                    D_("removing service data for %s!\n", buf + 1);
                    service_db_free(s);
                    write(sock, "1", 1);
                }
                else
                {
                    D_("Clearing complete db.\n");
                    service_db_clear_all();
                    write(sock, "1", 1);
                }
            }
            break;

        case 'x':                          /* restart initng */
            write(sock, "1", 1);            /* write status before, because it can't after */
            g.when_out = THEN_RESTART;
            stop_all();
            break;
#ifdef DEBUG
        case 'v':                          /* close /in and /out sockets */

            switch (g.verbose)
            {
                case 0:
                    g.verbose = 1;
                    write(sock, "1", 1);
                    break;
                case 1:
                    g.verbose = 0;
                    write(sock, "0", 1);
                    break;
                case 2:
                    g.verbose = 3;
                    write(sock, "3", 1);
                    break;
                case 3:
                    g.verbose = 2;
                    write(sock, "2", 1);
                    break;
                default:
                    g.verbose = 0;
                    W_("Unknow verbose id %i\n", g.verbose);
                    write(sock, "0", 1);
                    break;
            }
            W_("Verbose mode set to %i\n", g.verbose);
            break;
        case 'i':
            if (verbose_add(buf + 1))
                write(sock, "1", 1);
            else
                write(sock, "0", 1);
            break;
        case 'k':
            if (verbose_del(buf + 1))
                write(sock, "1", 1);
            else
                write(sock, "0", 1);
            break;
#endif
        case 'q':                          /* quit initng */
            write(sock, "1", 1);            /* write status before, because it can't after */
            g.when_out = THEN_QUIT;
            stop_all();
            break;
        case 'm':
            {
                m_h *mod = NULL;

                strcpy(send_buf, "modules: \n");

                while ((mod = initng_get_next_module(mod)))
                {
                    strcat(send_buf, mod->module_name);
                    strcat(send_buf, "\n");
                }
                send_buf[2001] = '\0';
                /*printf("m: \"%s\"\n", send_buf); */
                write(sock, send_buf, strlen(send_buf));
                break;
            }
        case 'o':
            {

                if (buf + 1 && strlen(buf + 1) > 3
                    && initng_load_module(buf + 1))
                {
                    write(sock, "1", 1);
                    break;
                }
                printf("bad request to ngc, load_module\n\n");
                write(sock, "0", 1);
                break;
            }
        case 'w':
            {

                if (buf + 1 && strlen(buf + 1) > 3
                    && initng_unload_module_named(buf + 1))
                {
                    write(sock, "1", 1);
                    break;
                }
                printf("bad request to ngc, unload_module\n\n");
                write(sock, "0", 1);
                break;
            }
        case 'p':                          /* print special service info or all to the initng console */
            {
                service_h *s;
                char *data = NULL;

                if (buf + 1 && (s = service_db_find_by_name(buf + 1)))
                {
                    data = service_db_print(s);
                }
                else
                {
                    data = service_db_print_all();
                }
                write(sock, data, strlen(data));
                free(data);
            }
            break;
        case 'n':                          /* send percent done */
            {
                char send[10];

                sprintf(send, "%i", active_db_percent_started());
                write(sock, send, 4);
            }
            break;
        case 'P':
            {                               /* set PID to in process_db */
                char *sname = NULL;
                int pid = 0;
                sname = initng_calloc(100, sizeof(char));
                sscanf(buf, "P%i:%s", &pid, sname);
                if (pid && sname)
                {
                    apt = active_db_find_by_name(sname);
                    if (apt && apt->name && apt->start_process)
                    {
                        D_("setting pid of %s from %i to %i!\n",
                           apt->name, apt->start_process->pid, pid);
                        apt->start_process->pid = pid;
                        if (active_db_is(&UP_WHEN_PID_SET, apt))
                            apt->a_status = RUNNING;
                        write(sock, "1", 1);
                    }
                    else
                    {
                        F_("did not found service %s to set pid %i to.",
                           sname, pid);
                        write(sock, "0", 1);
                    }
                }
                else
                {
                    F_("Bad control call trying to change pid\n");
                    write(sock, "0", 1);
                }
                free(sname);
                break;
            }
        case 'y':
            write(sock, "1", 1);
            stop_unneeded_services();
            break;

        default:
            F_("Invalid control key %c, string : %s", buf[0], buf);
            break;
    }
    write(sock, '\0', 1);
    return;
}

/* this adds the serversock to the global select */
static void addsock(void)
{
    D_("addsock %d\n", serversock);
    if (serversock >= 3)
        initng_add_hook(FDWATCHERS, &accepted_client, serversock);
    else
        F_("libngc2 trying to add a fdhook that is not >= 3\n");
}

/* called by fd hook, when data is no socket */
void accepted_client(void)
{
    int newsock;

    D_("Got here from fd hook.\n");
    /* we try to fix socket after every service start
       if it fails here chances are a user screwed it
       up, and we shouldn't manually try to fix anything. */
    if (serversock <= 0)
    {
        /* socket is invalid but we were called, call closesocket to make sure it wont happen again */
        F_("accepted client called with serversock %d, report bug\n",
           serversock);
        closesock();
        F_("Attempting to reopen socket\n");
        open_socket();
        return;
    }

    /* create a new socket, for reading */
    if ((newsock = accept(serversock, NULL, NULL)) > 0)
    {
        D_("read socket open, now reading\n");
        /* read the data, by the doread function */
        doread(newsock);
        /* clean up, and return */
        close(newsock);
        D_("Success!\n");
        return;
    }

    /* temporary unavaible */
    if ((errno == EAGAIN) || (errno == EWOULDBLOCK))
    {
        W_("errno = EAGAIN!\n");
        return;
    }

    /* This'll generally happen on shutdown, dont cry about it. */
    D_("Error accepting socket %d, %s\n", serversock, strerror(errno));
    closesock();
    return;
}

/* Send a ping to ourselfes to check if we're 100% ok. */
static int sendping()
{
    int client;
    struct sockaddr_un sockname;
    char buf[2];

    D_("Sending ping\n");

    /* Create the socket. */
    client = socket(PF_UNIX, SOCK_STREAM, 0);
    if (client < 0)
    {
        F_("Failed to init socket\n");
        return FALSE;
    }

    /* Bind a name to the socket. */
    sockname.sun_family = AF_UNIX;

    strcpy(sockname.sun_path, socket_filename);

    /* Try to connect */
    if (connect
        (client, (struct sockaddr *) &sockname,
         (strlen(sockname.sun_path) + sizeof(sockname.sun_family))) < 0)
    {
        close(client);
        return FALSE;
    }

    /* Write X to ping ourselfes */

    if (write(client, "X", 1) != 1)
    {
        F_("Could not send ping\n");
        close(client);
        return FALSE;
    }

    /* Accept "server side" */
    accepted_client();

    /* Note, this could hardlock, but I dont see how it can in any way since the write is ok. */
    if ((read(client, buf, 2) != 1) && buf[0] != 'Y')
    {
        F_("Could not recieve pong, buf[0] : %c\n", buf[0]);
        close(client);
        return FALSE;
    }
    close(client);

    D_("Got pong\n");
    return TRUE;
}

/* this will try to open a new socket */
static int open_socket()
{
    /*    int flags; */
    struct sockaddr_un serv_sockname;

    /* Make /dev/initng if it doesn't exist (try either way) */
    mkdir(SOCKET_ROOTPATH, S_IRUSR | S_IWUSR | S_IXUSR);

    /* chmod root path for root use only */
    if (chmod(SOCKET_ROOTPATH, S_IRUSR | S_IWUSR | S_IXUSR) == -1)
    {
        /* path doesn't exist, we dont have /dev yet. */
        if (errno == ENOENT || errno == EROFS)
            return (FALSE);

        F_("CRITICAL, failed to chmod %s, THIS IS A SECURITY PROBLEM.\n",
           SOCKET_ROOTPATH);
    }

    closesock();

    /* Create the socket. */
    serversock = socket(PF_UNIX, SOCK_STREAM, 0);
    if (serversock < 1)
    {
        F_("Failed to init socket (%s)\n", strerror(errno));
        serversock = 0;
        return (FALSE);
    }

    /* Set socket to non blocking mode */
    /*    flags = fcntl(serversock, F_GETFL);
       if (flags < 0)
       {
       F_("Failed to fcntl serversock\n");
       closesock();
       return (FALSE);
       }
       flags |= O_NONBLOCK;
       if (fcntl(serversock, F_SETFL, flags) < 0)
       {
       F_("Failed to set serversock O_NONBLOCK\n");
       closesock();
       return (FALSE);
       } */


    /* Bind a name to the socket. */
    serv_sockname.sun_family = AF_UNIX;

    strcpy(serv_sockname.sun_path, socket_filename);

    /* Remove old socket file if any */
    unlink(serv_sockname.sun_path);

    /* Try to bind */
    if (bind
        (serversock, (struct sockaddr *) &serv_sockname,
         (strlen(serv_sockname.sun_path) +
          sizeof(serv_sockname.sun_family))) < 0)
    {
        F_("Error binding to socket (errno: %d str: '%s')\n", errno,
           strerror(errno));
        closesock();
        unlink(serv_sockname.sun_path);
        return (FALSE);
    }

    /* chmod socket for root only use */
    if (chmod(serv_sockname.sun_path, S_IRUSR | S_IWUSR) == -1)
    {
        F_("CRITICAL, failed to chmod %s, THIS IS A SECURITY PROBLEM.\n",
           serv_sockname.sun_path);
        closesock();
        return (FALSE);
    }

    /* store sock_stat for checking if we need to recreate socket later */
    stat(serv_sockname.sun_path, &sock_stat);

    /* Listen to socket */
    if (listen(serversock, 5))
    {
        F_("Error on listen (errno: %d str: '%s')\n", errno, strerror(errno));
        closesock();
        unlink(serv_sockname.sun_path);
        return (FALSE);
    }


    /* att socket to initng select, with hook */
    addsock();

    /* Run check : */
    if (!sendping())
    {
        F_("Sendping check failed, ngc2 comunication not available (if you see this open a bug)\n");
        closesock();
        return (FALSE);
    }

    return (TRUE);
}

/* this will check socket, and reopen on failure */
static void check_socket()
{
    struct stat st;

    D_("Checking socket\n");

    /* Check if socket needs reopening */
    if (serversock <= 0)
    {
        D_("serversock not set, opening new socket.\n");
        open_socket();
        return;
    }

    /* stat the socket, reopen on failure */
    memset(&st, 0, sizeof(st));
    if (stat(socket_filename, &st) < 0)
    {
        W_("Stat failed! Opening new socket.\n");
        open_socket();
        return;
    }

    /* compare socket file, with the one that we know, reopen on failure */
    if (st.st_dev != sock_stat.st_dev || st.st_ino != sock_stat.st_ino
        || st.st_mtime != sock_stat.st_mtime)
    {
        F_("Invalid socket found, reopening\n");
        open_socket();
        return;
    }

    D_("Socket ok.\n");
    return;
}

/* this function, will make a check for socket, on every new service that goes up */
void service_status(active_h * service)
{
    /* only try open, when a service got up */
    if (service->a_status != DONE && service->a_status != RUNNING)
        return;
    D_("\n\nCHECK SOCKET HERE!\n\n");

    check_socket();
    /* Try ping ourself, return on success
       if (!sendping()) {
       W_("Sendping failed, reopening socket.\n");
       open_socket();
       }
     */

}

int module_init(void)
{
    D_("module_init(ngc2);\n");

    /* zero globals */
    serversock = 0;
    memset(&sock_stat, 0, sizeof(sock_stat));


    /* decide which socket to use */
    if (g.i_am_init)
        socket_filename = SOCKET_FILENAME_REAL;
    else
        socket_filename = SOCKET_FILENAME_TEST;

    /* do a check for an existing socket, otherwise it will open a new */
    /*check_socket(); */

    D_("adding hook, that will retry open ngc2.\n");
    initng_add_hook(WATCHERS, &service_status);

    D_("ngc2.so.0.0 loaded!\n");
    return (TRUE);
}


void module_unload(void)
{
    D_("module_unload(ngc2);\n");

    /* close open sockets */
    closesock();

    /* remove hooks */
    initng_del_hook(WATCHERS, &service_status);

    D_("ngc2.so.0.0 unloaded!\n");

}
