
/* Initng, a 6next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "../../src/initng.h"

#define _GNU_SOURCE
#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include "../../src/initng_toolbox.h"
#include "../../src/initng_global.h"
#include "../../src/service_db.h"


#include "print_service.h"


/* P R I N T   S E R V I C E */


static char *service_db_pr(s_entry * type, char *value)
{
    char *dup = NULL;
    int value_size = 0;

    D_(" (%s, %s)\n", type->opt_name, value);
    if (!value)
    {
        F_("empty value!\n");
        return (NULL);
    }

    value_size = strlen(value);

    if (value_size >= MAX_PRINT_LENGTH)
        dup =
            (char *)
            initng_calloc((strlen(type->opt_name) + MAX_PRINT_LENGTH + 15),
                          sizeof(char));
    else
        dup =
            (char *) initng_calloc((strlen(type->opt_name) + value_size + 15),
                                   sizeof(char));


    strcpy(dup, " @@ ");
    strcat(dup, type->opt_name);
    strcat(dup, ":\t\"");

    if (value_size >= MAX_PRINT_LENGTH)
        strncat(dup, value, MAX_PRINT_LENGTH);
    else
        strcat(dup, value);


    /* do ending */
    strcat(dup, "\"\n");

    /*D_("returning \"%s\"\n", dup); */
    return (dup);
}

static char *pr_int(s_entry * type, int value)
{
    char *to_ret = NULL;

    to_ret =
        (char *) initng_calloc((strlen(type->opt_name) + 20), sizeof(char));
    sprintf(to_ret, " @@ %s:\t\"%i\"\n", type->opt_name, value);
    return (to_ret);
}

/* some stdout */
char *service_db_print(service_h * s)
{
    char *p = NULL;
    unsigned int to_alloc;

    /*data path */
    s_data *tmp = s->data;

    assert(s);
    assert(s->name);

    /* try calculate out how mutch memory we wanna allocate
       that we will send */
    to_alloc = strlen(s->name) + 55;

    while (tmp && tmp->type)
    {
        if ((tmp->type->opt_type == STRING ||
             tmp->type->opt_type == STRINGS ||
             tmp->type->opt_type == SCRIPT) && tmp->t.s)
        {
            to_alloc = to_alloc + strlen(tmp->t.s) + 20;
        }
        else
        {
            /* else do a guess */
            to_alloc = to_alloc + 30;
        }
        tmp = tmp->next;
    }

    D_("Allocating %i chars for print out!\n", to_alloc);

    /* this allocation will contain the complete
       request answer of servie_db_print ! */
    p = (char *) initng_calloc(to_alloc, sizeof(char));
    switch (s->type)
    {
        case TYPE_SERVICE:
            strcpy(p, "\n#   service \"");
            strcat(p, s->name);
            strcat(p, "\"\n");
            break;
        case TYPE_CLASS:
            strcpy(p, "\n#   class   \"");
            strcat(p, s->name);
            strcat(p, "\"\n");
            break;
        case TYPE_RUNLEVEL:
            strcpy(p, "\n#  runlevel \"");
            strcat(p, s->name);
            strcat(p, "\"\n");
            break;
        default:
            strcpy(p, "\n#   unknown \"");
            strcat(p, s->name);
            strcat(p, "\"\n");
            break;
    }
    strcat(p, "    ------------------------ \n");


    D_("so far it is %i chars!\n", strlen(p));
    tmp = s->data;
    while (tmp)
    {
        if (tmp->type)
        {
            char *got = NULL;

            if (tmp->type->opt_type == STRING ||
                tmp->type->opt_type == STRINGS ||
                tmp->type->opt_type == SCRIPT)
                got = service_db_pr(tmp->type, tmp->t.s);
            else if (tmp->type->opt_type == INT)
                got = pr_int(tmp->type, tmp->t.i);

            if (got)
            {
                if ((strlen(got)) < (to_alloc - strlen(p)))
                    strcat(p, got);
                else
                    F_("allocated to less!, want %i, have %i  cant output!\n",
                       strlen(got), (to_alloc - strlen(p)));

                free(got);
            }
        }
        tmp = tmp->next;
    }
    D_("Total sent to ngc: \"%s\"\n", p);
    return (p);
}

/* Walk through every service and print it all */
char *service_db_print_all(void)
{
    char *t = NULL;
    char *p = NULL;
    p = initng_calloc(10000, sizeof(char));
    strcpy(p, "Full service print:\n");
    service_h *next = NULL;

    D_("print_all():\n");
    while ((next = service_db_get_next(next)))
    {
        t = service_db_print(next);
        strcat(p, t);
        free(t);
    }
    /*printf("Full is:\"%s\"\n",p); */
    return (p);
}
