
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "../../src/initng.h"

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <signal.h>
#include <errno.h>
#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_common.h"
#include "../../src/initng_plugin_hook.h"

#include "initng_pidfile.h"

s_entry PIDFILE = { "pid_file", STRING, 8 };
int handle_killed(active_h * s, process_h * p)
{
    int fd = 0;
    int len = 0;
    int newpid = 0;
    char buf[21];

    char *pidfile = active_db_get_string(&PIDFILE, s);

    /* We only override start behavior, and only if pidfile is set */
    if ((!pidfile) || (!s->start_process))
        return (FALSE);

    pidfile = fix_variables(pidfile, s);

    D_("Handle_killed for %s (%s)\n", s->name, pidfile);


    /* open pid file */
    fd = open(pidfile, O_RDONLY);

    /* Warn here? */
    if (fd == -1)
        return (FALSE);

    /* Read data from buffer */
    len = read(fd, buf, 20);
    close(fd);
    if (len < 1)
    {
        F_("Read 0 from %s\n", pidfile);
        free(pidfile);
        return (FALSE);
    }

    /* remove last newline */
    if (buf[len - 1] == '\n')
        buf[len - 1] = 0;
    else
        buf[len] = 0;

    /* Try to convert pid to int */
    newpid = atoi(buf);
    if (newpid < 2)
    {
        F_("Failed to convert pid %s (%s) to int\n", buf, pidfile);
        free(pidfile);
        return (FALSE);
    }

    /* Check if the old pid == newpid) */
    if (newpid == p->pid)
    {
        D_("newpid == p->pid");
        free(pidfile);
        return (FALSE);
    }


    /* See if it's a valid pid */
    if (kill(newpid, 0) < 0 && (errno == ESRCH))
    {
        F_("Pid %d found (%s), but is dead\n", newpid, pidfile);
        free(pidfile);
        return (FALSE);
    }

    /* Finally set it!\n */
    D_("Setting pid for %s from %d to %d\n", s->name, p->pid, newpid);
    p->pid = newpid;
    mark_service(s, RUNNING);

    free(pidfile);
    return (TRUE);
}

int module_init(void)
{
    D_("module_init();\n");

    initng_sdt_add(&PIDFILE);

    initng_add_hook(HANDLE_KILLED, &handle_killed);

    return (TRUE);
}

void module_unload(void)
{
    D_("module_unload();\n");
    initng_sdt_del(&PIDFILE);
    initng_del_hook(HANDLE_KILLED, &handle_killed);
}
