/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 neuron <neuron@hollowtube.mine.nu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include "../../src/initng.h"


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <fcntl.h>
#include "../../src/initng_handler.h"
#include "../../src/initng_global.h"
#include "../../src/initng_plugin_hook.h"

#include "initng_readahead.h"

/* do the actual readahead */
void doreadahead(char *file)
{
    int fd;
    struct stat st;

    if (file[0] == '#')
        return;
    D_("readahead %s\n", file);
    fd = open(file, O_RDONLY);
    if ((fd < 0) || (fstat(fd, &st) < 0))
        return;
    readahead(fd, (loff_t) 0, (size_t) st.st_size);
    close(fd);
}

/* Load readahead files in /etc/initng/readahead.d */
void startreadahead()
{
    struct dirent **namelist;
    struct stat st;
    int i, fd, pos, oldpos;
    char filetoopen[256];
    char *buffer;

    D_("Starting readahead\n");

    /* todo improve this, scandir is not POSIX, but neither is readahead.. */
    i = scandir(READAHEAD_DIR, &namelist, 0, alphasort);
    if (i == -1)
    {
        F_("Failed to open dir %s\n", READAHEAD_DIR);
        return;
    }
    while (i--)
    {
        if ((strcmp(namelist[i]->d_name, ".") == 0)
            || (strcmp(namelist[i]->d_name, "..") == 0))
            continue;
        /* Open file and get length */
        strcpy(filetoopen, READAHEAD_DIR);
        strcat(filetoopen, namelist[i]->d_name);
        D_("Loading and caching files in %s\n", filetoopen);
        fd = open(filetoopen, O_RDONLY);
        if ((fd < 0) || (fstat(fd, &st) < 0))
        {
            F_("Failed to open %s\n", filetoopen);
            free(namelist[i]);
            continue;
        }

        /* Make a buffer and read entire file */
        if ((buffer = malloc(st.st_size + 1)) == NULL)
        {
            F_("Malloc file %s size %d failed!\n", namelist[i]->d_name,
               (int) st.st_size);
            free(namelist[i]);
            close(fd);
            continue;
        }
        pos = read(fd, buffer, st.st_size);
        if (pos != st.st_size)
        {
            F_("Failed to read file %s\n", namelist[i]->d_name);
            free(namelist[i]);
            free(buffer);
            close(fd);
            continue;
        }
        /* We "need" trailing slash, and we malloc size+1 */
        buffer[pos] = '\n';
        buffer[pos++] = '\0';
        close(fd);

        /* Read line by line and send to doreadahead */
        pos = 0;
        oldpos = 0;
        while (buffer[pos++] != '\0')
        {
            if (buffer[pos] == '\n')
            {
                buffer[pos] = '\0';
                doreadahead(&buffer[oldpos]);
                /* doing pos ++ here so the while loop dont instantly run into \0 */
                pos++;
                oldpos = pos;
            }
        }
        free(namelist[i]);
    }
    free(namelist);
}

void service_status(active_h * service)
{
    /* check for DONE and STOPPING of fake service readahead */
    if (service->a_status == DONE)
        if (strcmp(service->name, "plugin/readahead") == 0)
            startreadahead();
}

int module_init(void)
{
    D_("module_init();\n");
    return (initng_add_hook(WATCHERS, &service_status));
}

void module_unload(void)
{
    D_("module_unload();\n");
    initng_del_hook(WATCHERS, &service_status);
}
