
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef ACTIVE_DB_H
#define ACTIVE_DB_H

#include <sys/types.h>
#include <unistd.h>
#include "service_db.h"
#include "process_db.h"

#include "active_status.h"
#include "struct_data.h"

#define MARK_NEEDED 1
#define MARK_RESTARTING 2

typedef struct active_type active_h;
struct active_type
{
    char *name;
    service_h *from_service;
    e_a_status a_status;
    time_t time_got_status;
    active_h **depends;
    process_h *start_process;
    process_h *stop_process;
    s_data *data;
    int m;
};

active_h *active_db_find_by_name(const char *service);
active_h *active_db_find_by_pid(pid_t pid);
active_h *active_db_find_by_service_h(service_h * service);
const char *active_db_get_status_string(e_a_status status);
void active_db_change_service_h(service_h * from, service_h * to);
int active_db_add(active_h ** new_a);
int active_db_count(e_a_status type);
void active_db_compensate_time(time_t skew);
void active_db_free(active_h * pf);
void active_db_free_all(void);
void active_db_reset(active_h * res);
void active_db_init(void);
active_h *active_db_next(active_h * current);
active_h *active_db_next_dep(active_h * dep, active_h * current);
int active_db_new(active_h ** new_a, const char *name);

int active_db_percent_started(void);
int active_db_percent_stopped(void);

int active_db_get_int(s_entry * type, active_h * from_active);
char *active_db_get_string(s_entry * type, active_h * from_active);
char *active_db_get_next_string(s_entry * type, active_h * from_active,
                                char *last);
void active_db_set_string(s_entry * type, active_h * from_active,
                          char *string);
int active_db_is(s_entry * type, active_h * from_active);
int active_db_count_type(s_entry * type, active_h * from_active);
void active_db_set(s_entry * type, active_h * from_active);
void active_db_remove(s_entry * type, active_h * from_active);

void active_db_unmark_all(int mark_type);
void active_db_mark(active_h * s, int mark_type);
void active_db_unmark(active_h * s, int mark_type);

#define active_db_is_m(S, M) S->m/M

#endif
