
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "initng.h"

#define _GNU_SOURCE
#include <stdlib.h>                         /* malloc() free() exit() */
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>

#include "initng_error.h"

#include "initng_global.h"
#include "initng_toolbox.h"
#include <assert.h>

const char *last_file = NULL;
const char *last_func = NULL;

#ifdef DEBUG
static void verbose_print(void)
{
    int i;

    W_("This words will i look for in debug: ");
    for (i = 0; i < MAX_VERBOSES; i++)
        if (g.verbose_this[i])
            printf("\"%s\" ", g.verbose_this[i]);
    printf("\n");
}

int verbose_add(const char *string)
{
    int i = 0;

    if (g.verbose == 1)
        g.verbose = 3;
    else
        g.verbose = 2;


    while (g.verbose_this[i] && i < MAX_VERBOSES)
        i++;

    if (i >= MAX_VERBOSES - 1)
    {
        F_("Can't add another \"%s\" debug trace\n", string);
        return (FALSE);
    }

    g.verbose_this[i] = i_strdup(string);

    verbose_print();

    if (g.verbose_this[i])
        return (TRUE);
    return (FALSE);
}

int verbose_del(const char *string)
{
    int i;
    int t = FALSE;

    for (i = 0; i < MAX_VERBOSES; i++)
        if (g.verbose_this[i] && strcasecmp(g.verbose_this[i], string) == 0)
        {
            free(g.verbose_this[i]);
            g.verbose_this[i] = NULL;
            t = TRUE;
        }
    verbose_print();
    return (t);
}
#endif

int print_error(e_mt mt, const char *file, const char *func, int line,
                const char *format, ...)
{
    int done;

    assert(file);
    assert(func);

#ifdef DEBUG
    if (mt == MSG_VERBOSE)
    {
        int i;

        if (g.verbose == 1)
            goto yes;
        if (g.verbose == 2 || g.verbose == 3)
            for (i = 0; i < MAX_VERBOSES; i++)
                if (g.verbose_this[i])
                {
                    if (g.verbose_this[i][0] == '%'
                        && (g.verbose_this[i] + 1))
                    {
                        if (strcasestr(file, (g.verbose_this[i]) + 1) ||
                            strcasestr(func, (g.verbose_this[i]) + 1))
                        {
                            return (TRUE);
                        }
                    }
                    else
                    {
                        if (strcasestr(file, g.verbose_this[i]) ||
                            strcasestr(func, g.verbose_this[i]))
                        {
                            goto yes;
                        }
                    }
                }
        if (g.verbose == 3)
            goto yes;
        /* else */
        return (TRUE);
    }
  yes:
#endif
    if (last_file != file || last_func != func)
    {
        fprintf(stderr, "\n\n ** \"%s\", %s():\n", file, func);
    }

    last_file = file;
    last_func = func;

    switch (mt)
    {
        case MSG_FAIL:
            fprintf(stderr, " -- line:%i FAIL\t", line);
            break;
        case MSG_WARN:
            fprintf(stderr, " -- line:%i WARN\t", line);
            break;
        case MSG_VERBOSE:
            fprintf(stderr, " -- line:%i\t", line);
            break;
    }


    va_list arg;

    va_start(arg, format);

    done = vfprintf(stderr, format, arg);

    va_end(arg);

    /*
       if (mt == MSG_FAIL)
       {
       fprintf(stderr, "\n **  Sleeping 3 sec ...  **\n");
       sleep(3);
       } */
    return (done);
}
