
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include "initng.h"
#include "initng_global.h"
#include "initng_load_module.h"
#include "initng_toolbox.h"

s_global g;

void initng_new(void)
{
    g.Argv = NULL;
    g.Argc = 0;
    g.config_allocated = 0;
    g.active_allocated = 0;
    g.interrupt = FALSE;
    g.sys_state = STATE_NULL;
    g.runlevel = NULL;

    /* clear g.modules and g.parsers */
    initng_load_module_init();

    /* clear and allocate service database */
    service_db_init();

    /* clear and allocate active database */
    active_db_init();

    /* clear and allocate history database */
    history_db_init();

    /* Add defaults (static) to data_types */
    initng_sdt_add_defaults();


#ifdef DEBUG
    g.verbose = 0;
    {
        int i;

        for (i = 0; i < MAX_VERBOSES; i++)
            g.verbose_this[i] = NULL;
    }
#endif
}

void initng_free(void)
{
    history_free_all();                     /* flush history log db */
    active_db_free_all();                   /* clean process_db */
    service_db_free_all();                  /* clean service db */

    initng_sdt_free_all();
    if (g.runlevel)
        free(g.runlevel);

    /* clear all g.Argv[i] */
    {
        int i;

        for (i = 0; g.Argv[i]; i++)
            if (g.Argv[i])
                free(g.Argv[i]);
        g.Argc = 0;
    }
}
