
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "initng.h"

#include <time.h>                           /* time() */
#include <fcntl.h>                          /* fcntl() */
#include <sys/un.h>                         /* memmove() strcmp() */
#include <sys/wait.h>                       /* waitpid() sa */
#include <linux/kd.h>                       /* KDSIGACCEPT */
#include <sys/ioctl.h>                      /* ioctl() */
#include <stdio.h>                          /* printf() */
#include <stdlib.h>                         /* malloc() free() exit() */
#include <sys/reboot.h>                     /* reboot() RB_DISABLE_CAD */
#include <assert.h>
#include <errno.h>

#include "initng_global.h"

#include "active_db.h"
#include "initng_toolbox.h"
#include "initng_main.h"
#include "initng_execute.h"
#include "initng_common.h"
#include "initng_load_module.h"

#include "initng_handler.h"
#include "initng_kill_handler.h"

#include "static_data_id.h"

/* function defines */
static int load_all_deps(active_h * process_starting);
static int try_to_start_service(active_h * service_to_start);
static int try_to_stop_service(active_h * service_to_stop);
static void set_up_or_done(active_h * service_to_start);





    /* will try start processes waiting for dependencies to go up */
void try_start_wait_deps(void)
{
    active_h *tmp = NULL;

    D_("try_start_wait_deps(void);\n");

    /* walk through all active services */
    while ((tmp = active_db_next(tmp)))
    {
        D_("try_start_wait_deps(): re-trying with %s.\n", tmp->name);

        /* Look at status */
        switch (tmp->a_status)
        {
            case WAITING_FOR_STOP_DEP:
                /* if succeeded to stop some, remember this */
                try_to_stop_service(tmp);
                break;
            case WAITING_FOR_START_DEP:
                /* if succeeded to start some, remember this too */
                /* If start dependencies are suddenly met, but we're stopping the system try to stop */
                if (g.sys_state == STATE_STOPPING)
                    try_to_stop_service(tmp);
                else
                    try_to_start_service(tmp);
                break;
            default:
                break;
        }
    }

}



    /* Load dependencies */
static int load_all_deps(active_h * service_starting)
{
    char *dep = NULL;
    int a_deps = 0;
    int deps = 0;

    assert(service_starting);
    D_("load_all_deps(%s);\n", service_starting->name);

    /* Check if we've already loaded dependencies */
    if (service_starting->depends)
        return (TRUE);

    /* if there are no dependencies don't do anything */
    if (!(a_deps = active_db_count_type(&NEED, service_starting)))
    {
        D_("load_all_deps(%s): service has no dependencies\n",
           service_starting->name);
        service_starting->depends = NULL;
        return (TRUE);
    }

    D_("allocating for %i strings on %s.\n", a_deps, service_starting->name);
    service_starting->depends =
        (active_h **) initng_calloc(a_deps + 1, sizeof(active_h *));
    while ((dep = active_db_get_next_string(&NEED, service_starting, dep)))
    {
        D_("a dep is:\"%s\" #%i.\n", dep, deps);
        load_to_active(&service_starting->depends[deps], dep);
        deps++;
    }
    service_starting->depends[deps] = NULL;
    D_("load_all_deps(%s): Returning success.\n", service_starting->name);
    return (TRUE);

}

    /* this function will send term signal to a service if it was marked 30 seconds or more */
void term_stopping_services(void)
{
    active_h *to_term = NULL;

    while ((to_term = active_db_next(to_term)))

        if (to_term &&
            to_term->a_status == STOPPING &&
            to_term->time_got_status &&
            to_term->start_process && to_term->start_process->pid)
        {
            /*printf("tome_got_status=%i, time now=%i\n",(int) to_term->time_got_status, (int)time(0)); */
            /* todo, if kill(process,0) = dead then handle_killed_by_pid without warning and timeout. */

            if (kill(to_term->start_process->pid, 0) < 0 && (errno == ESRCH))
            {
                /* This process is already dead */
                F_("process %s pid %d is already dead, trying to fix.\n",
                   to_term->name, to_term->start_process->pid);
                handle_killed_by_pid(to_term->start_process->pid, 1);
                continue;
            }
            if (to_term->time_got_status + 9 < time(0))
            {
                W_("Warning, terminating %s on pid %i!\n", to_term->name,
                   to_term->start_process->pid);
                /* Uhm, this doesnt work : kill(-to_term->start_process->pid, SIGKILL); */
                kill(to_term->start_process->pid, SIGKILL);
                /* sigchld might not trigger here, for example if we're using an invalid pidfile (pid = not ours?) */
                /*                handle_killed_by_pid(to_term->start_process->pid, 1); */
                continue;
            }
            alarm(10);                      /* if there are stopping processes, schedule a new alarm */
        }

}

    /* M A R K    S T U F F */

    /* When this function is called, the service is marked for starting */
int start_service(active_h * service_to_start)
{
    assert(service_to_start);
    assert(service_to_start->name);

    /* check system state, if we can launch. */
    if (g.sys_state != STATE_STARTING && g.sys_state != STATE_UP)
    {
        F_("Cant start service, when system status is: %i !\n", g.sys_state);
        return (FALSE);
    }

    /* it might actually already be up */
    if (service_to_start->a_status == DONE ||
        service_to_start->a_status == RUNNING)
        return (TRUE);

    /* if service has not started stopping yet, it can be set to up directly */
    if (service_to_start->a_status == WAITING_FOR_STOP_DEP ||
        service_to_start->a_status == STOP_DEP_MET)
    {
        set_up_or_done(service_to_start);
        return (TRUE);
    }

    /* STEP 2, service must be in one of these statuses to start */
    if (service_to_start->a_status != NONE &&
        service_to_start->a_status != STOPPED &&
        service_to_start->a_status != RESPAWNING &&
        service_to_start->a_status != RESTARTING)
        return (FALSE);

    /* STEP 3, mark service starting */
    mark_service(service_to_start, LOADING);

    /* if there are dependencies load them too */
    if (active_db_is(&NEED, (service_to_start)))
    {
        if (!load_all_deps(service_to_start))
        {
            F_("load_to_active(%s): dependency refuses to load!\n",
               service_to_start->name);
            mark_service(service_to_start, FAIL_STARTING);
            return (FALSE);
        }
    }

    /* STEP 5, try mark-start all dependencies */
    if (service_to_start->depends)
    {
        int i;

        /* walk through and launch all */
        for (i = 0; service_to_start->depends[i]; i++)
            start_service(service_to_start->depends[i]);
        /* settle back, and wait for deps */
    }

    /* make sure try_to_start_wait_deps is run on main loop */
    g.interrupt = TRUE;

    /* mark it WAITING_FOR_START_DEP and wait */
    mark_service(service_to_start, WAITING_FOR_START_DEP);
    return (TRUE);
}

    /* S T O P     S E R V I C E */
int stop_service(active_h * service_to_stop)
{
    active_h *current = NULL;

    assert(service_to_stop);

    D_("stop_service(%s);\n", service_to_stop->name);

    /* if service is alredy stoped, return */
    if (service_to_stop->a_status == STOPPED)
        return (TRUE);

    /* this can be a cirular dep, if then return */
    if (service_to_stop->a_status == STOP_MARKED ||
        service_to_stop->a_status == WAITING_FOR_STOP_DEP)
        return (TRUE);

    /* if service are waiting to start, there is no problem to shut it down directly */
    if (service_to_stop->a_status == WAITING_FOR_START_DEP ||
        service_to_stop->a_status == START_DEP_MET)
    {
        mark_service(service_to_stop, STOP_DEP_MET);
        return (TRUE);
    }

    /* check if service status, is really up */
    if (service_to_stop->a_status != DONE &&
        service_to_stop->a_status != RUNNING)
        W_("Warning, setting service %s, status %s, to STOP_MARKED!\n",
           service_to_stop->name,
           active_db_get_status_string(service_to_stop->a_status));

    /* set stopping */
    mark_service(service_to_stop, STOP_MARKED);

    /* mark all services depending on this service stop */
    while ((current = active_db_next_dep(service_to_stop, current)))
    {
        D_("%s have to stop %s.\n", service_to_stop->name, current->name);
        stop_service(current);
    }

    /* make sure, main loop tries to start WAITING_FOR_STARTDEP */
    g.interrupt = TRUE;

    /* Mark this service, that is is waiting for its stop deps to stop */
    mark_service(service_to_stop, WAITING_FOR_STOP_DEP);

    return (TRUE);
}

int restart_service(active_h * service_to_restart)
{
    active_db_mark(service_to_restart, MARK_RESTARTING);
    return (stop_service(service_to_restart));
}


/* loads this service and all dependendcies */
active_h *load_new_service(active_h * to_load)
{
    int i;

    D_("Loading %s\n", to_load->name);

    /* First load all our dependencies */
    if (!load_all_deps(to_load))
    {
        F_("load_to_active(%s): dependency refuses to load!\n",
           to_load->name);
        mark_service(to_load, FAIL_STARTING);
        return (FALSE);
    }

    /* Then scan through them and do load_new_service on them */
    if (to_load->depends)
    {
        /* walk through and launch all */
        for (i = 0; to_load->depends[i]; i++)
            if (!load_new_service(to_load->depends[i]))
                return (FALSE);
    }
    return to_load;
}

/* Load new process named */
active_h *load_new_service_named(const char *service)
{
    active_h *to_load = NULL;

    assert(service);
    load_to_active(&to_load, service);
    if (to_load)
    {
        return load_new_service(to_load);
    }
    return (FALSE);
}

    /* start new process named */
int start_new_service_named(const char *service)
{
    active_h *to_start = NULL;

    assert(service);
    load_to_active(&to_start, service);

    if (to_start)
        return (start_service(to_start));

    return (FALSE);
}

    /* enter a new runlevel */
int stop_all(void)
{
    active_h *ser = NULL;

    set_sys_state(STATE_STOPPING);
    D_("stop_all();\n");
    while ((ser = active_db_next(ser)))
    {
        if (ser)
            stop_service(ser);
    }
    return (TRUE);
}





/* These two functions will stop services that are not dependencies 
   of the current runlevel */
static void mark_deps(active_h * h)
{
    int i;

    active_db_mark(h, MARK_NEEDED);
    if (h->depends)
        for (i = 0; h->depends[i]; i++)
            mark_deps(h->depends[i]);
}
void stop_unneeded_services(void)
{
    active_h *s = NULL;

    /* Remove mark from all active services */
    active_db_unmark_all(MARK_NEEDED);

    /* Recursively mark all dependencies of the current runlevel */
    s = active_db_find_by_name(g.runlevel);
    if (!s)
        return;
    mark_deps(s);

    /* Now rewalk the tree, stopping anything not marked as needed */
    s = NULL;
    while ((s = active_db_next(s)))
    {
        if (!active_db_is_m(s, MARK_NEEDED))
        {
            stop_service(s);
        }
    }
}



    /* D O    S T U F F */


    /* This is the function that really launches the start-marked services if all deps are up */
static int try_to_start_service(active_h * service_to_start)
{
    assert(service_to_start);

    D_("try_to_start_service(%s): status: \"%s\"\n",
       service_to_start->name,
       active_db_get_status_string(service_to_start->a_status));

    if (g.sys_state != STATE_STARTING && g.sys_state != STATE_UP)
    {
        F_("Cant start service, when system status is: %i !\n", g.sys_state);
        return (FALSE);
    }

    /* check if all deps is up = DONE or RUNNING */
    if (service_to_start->depends)
    {
        int i;

        for (i = 0; service_to_start->depends[i]; i++)
            if ((service_to_start->depends[i]->a_status != DONE) &&
                (service_to_start->depends[i]->a_status != RUNNING))
            {
                D_(" \"%s\" wait for --> \"%s\"(s:%i).\n",
                   service_to_start->name,
                   service_to_start->depends[i]->name,
                   service_to_start->depends[i]->a_status);
                return (FALSE);
            }
    }

    /* set status to START_DEP_MET */
    mark_service(service_to_start, START_DEP_MET);


    /* some plugins maybe don't think that all deps are met */
    if (service_to_start->a_status != START_DEP_MET)
        return (FALSE);

    /* check if it has a start process */
    if (service_to_start->start_process)
    {
        F_("try_to_start_service(%s): Function already has a start process! Aborting\n", service_to_start->name);
        return (FALSE);
    }

    /* F I N A L L Y   S T A R T */

    switch (start_launch(service_to_start))
    {
        case FALSE:
            D_("Done direct!\n");
            /* D O N E - D I R E C T */
            mark_service(service_to_start, DONE);
            if (service_to_start->a_status == FAIL_STARTING)
                return (FALSE);
            return (TRUE);                  /* return with status */
        case FAIL:
            F_("Did not find START file!\n");
            mark_service(service_to_start, FAIL_STARTING);
            return (FALSE);                 /* return with status */

    }

    /* make sure there are a start process */
    if (!service_to_start->start_process)
    {
        mark_service(service_to_start, FAIL_STARTING);
        return (FALSE);                     /* return with status */
    }

    D_("Launch Started.\n");
    set_up_or_done(service_to_start);
    if (service_to_start->a_status == FAIL_STARTING)
        return (FALSE);
    return (TRUE);
}

    /* try to find out if service is up or done */
static void set_up_or_done(active_h * service_to_start)
{
    /* S E T   A   S T A T U S */

    /* if marked that service up when done or pid set, mark this
       service as starting */
    if (active_db_is(&UP_WHEN_DONE, service_to_start) ||
        active_db_is(&UP_WHEN_PID_SET, service_to_start))
    {
        D_("Found UP_WHEN_DONE or UP_WHEN_PID_SET, seting status to running.\n");
        mark_service(service_to_start, STARTING);
        return;
    }


    /* if UP_WHEN_STARTED set, set status and return */
    if (active_db_is(&UP_WHEN_STARTED, service_to_start) ||
        active_db_is(initng_sdt_find("daemon"), service_to_start))
    {
        D_("Found UP_WHEN_STARTED or DAEMON, seting status to running.\n");
        mark_service(service_to_start, RUNNING);
        return;
    }


    /* by default, return service starting */
    D_("defaulting to STARTING\n");
    mark_service(service_to_start, STARTING);
    return;                                 /* return with status */
}


    /* do stop service */
static int try_to_stop_service(active_h * service_to_stop)
{
    active_h *current = NULL;

    assert(service_to_stop);

    D_("try_to_stop_service(%s): status :\"%s\"\n",
       service_to_stop->name,
       active_db_get_status_string(service_to_stop->a_status));

    /* if there are services depending on this one still running, return false and still try */
    while ((current = active_db_next_dep(service_to_stop, current)))
    {
        if (current->a_status == DONE ||
            current->a_status == RUNNING ||
            current->a_status == STOP_MARKED ||
            current->a_status == STOPPING ||
            current->a_status == WAITING_FOR_STOP_DEP ||
            current->a_status == STOP_DEP_MET)
            return (FALSE);                 /* wait */
    }

    /* ok, stopping deps are met */
    mark_service(service_to_stop, STOP_DEP_MET);

    /* and hope that plugins think so */
    if (service_to_stop->a_status != STOP_DEP_MET)
        return (FALSE);

    /* check if it has a stop process */
    if (service_to_stop->stop_process)
    {
        F_("try_to_stop_service(%s): Function already have a stop process! Aborting!\n", service_to_stop->name);
        mark_service(service_to_stop, FAIL_STOPPING);
        return (FALSE);
    }

    /* F I N A L L Y   S T O P */
    mark_service(service_to_stop, STOPPING);

    /* launch stop service */
    switch (stop_launch(service_to_stop))
    {
        case FAIL:
            F_("try_to_stop_service(%s): fail launch stop!\n",
               service_to_stop->name);
            mark_service(service_to_stop, FAIL_STOPPING);
            return (FALSE);
        default:
            break;
    }


    /* if we got a stop process with a pid */
    if (service_to_stop->stop_process && service_to_stop->stop_process->pid)
    {
        alarm(10);                          /* the alarm will try to kill stopping services */
        return (TRUE);                      /* return from here */
    }

    /* T R Y 2 , Kill start_process directly */
    if (service_to_stop->start_process
        && service_to_stop->start_process->pid > 0)
    {
        D_("try_to_stop(%s): REAL KILLING PID %i!\n",
           service_to_stop->name, service_to_stop->start_process->pid);

        /* Uhm, this doesnt work : kill(-service_to_stop->start_process->pid, SIGTERM); */
        kill(service_to_stop->start_process->pid, SIGTERM);

        alarm(10);                          /* the alarm will try to kill stopping services */
        /*kill(service_to_stop->start_process->pid,SIGKILL); */
        return (TRUE);
    }
    /* if this is a empty dep service we show up here */

    /* remember this in history */
    mark_service(service_to_stop, STOPPED);

    /* check if this service is restarting */
    if (active_db_is_m(service_to_stop, MARK_RESTARTING))
    {
        mark_service(service_to_stop, RESTARTING);
        active_db_unmark(service_to_stop, MARK_RESTARTING);
        start_service(service_to_stop);
        return (TRUE);
    }


    /* free service, and forget */
    active_db_free(service_to_stop);

    return (TRUE);
}
