#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <dlfcn.h>
#include <assert.h>
#include <errno.h>
#include <string.h>
#include <fcntl.h>                          /* fcntl() */
#include "initng.h"
#include "service_db.h"
#include "initng_load_module.h"
#include "initng_toolbox.h"
#include "initng_global.h"
#include "initng_signal.h"

#include "initng_plugin_callers.h"
#include "initng_plugin_type.h"

/* P L U G I N    C A L L E R S */

/* if plugin returns <0 , it will directly stop, returning the value */
/* if plugin returns 0  , it will continue with next plugin */
/* if plugin returns 1  , it will be increment the return value, and take next plugin */
/* if plugin returns >1 , it will directly stop, return the value */
int initng_plugin_call(e_ptype type, ...)
{
    int i;
    int tmp = 0;
    int to_ret = 0;
    va_list list;

    va_start(list, type);

    for (i = 0; i < MAX_CALLERS; i++)
    {
        if (g.callers[i].ptype == type)
        {
            tmp = (*g.callers[i].c.hook) (list);
            if (tmp < 0)
            {                               /* -1 */
                va_end(list);
                return (tmp);
            }
            if (tmp > 1)
            {                               /* +2 */
                va_end(list);
                return (tmp);
            }
            if (tmp == 1)
                to_ret = 1;
        }
    }
    va_end(list);
    return (to_ret);
}

/* this function returns first pointer got */
void *initng_plugin_pcall(e_ptype type, ...)
{
    int i;
    void *data = NULL;
    va_list list;

    va_start(list, type);

    for (i = 0; i < MAX_CALLERS; i++)
    {
        if (g.callers[i].ptype == type)
        {
            if ((data = (*g.callers[i].c.phook) (list)))
            {
                va_end(list);
                return (data);
            }
        }
    }
    va_end(list);
    return (NULL);
}

int initng_plugin_launch_start(active_h * s)
{
    int i;
    int to_ret;

    for (i = 0; i < MAX_CALLERS; i++)
    {
        if (g.callers[i].ptype == STARTL)
        {
            if (!(to_ret = (*g.callers[i].c.launcher) (s)) == FALSE)
            {
                return (to_ret);
            }
        }
    }
    return (FALSE);
}

int initng_plugin_handle_killed(active_h * s, process_h * p)
{
    int i;

    for (i = 0; i < MAX_CALLERS; i++)
    {
        if (g.callers[i].ptype == HANDLE_KILLED)
        {
            if (((*g.callers[i].c.handle_killed) (s, p)) == TRUE)
                return (TRUE);
        }
    }
    return (FALSE);
}

int initng_plugin_after_fork(active_h * s, process_h * p)
{
    int i;
    int to_ret;

    assert(s);
    assert(p);
    D_("FORK IS DONE (%s)\n", s->name);

    for (i = 0; i < MAX_CALLERS; i++)
    {
        if (g.callers[i].ptype == A_FORK)
        {
            if ((to_ret = (*g.callers[i].c.af_launcher) (s, p)) == FALSE)
            {
                F_("Some after_fork plugin failed!\n");
                return (FALSE);
            }
        }
    }
    return (TRUE);
}

int initng_plugin_launch_stop(active_h * s)
{
    int i;
    int to_ret;

    for (i = 0; i < MAX_CALLERS; i++)
    {
        if (g.callers[i].ptype == STOPL)
        {
            if (!(to_ret = (*g.callers[i].c.launcher) (s)) == FALSE)
            {
                return (to_ret);
            }
        }
    }
    return (FALSE);
}

void initng_plugin_main(void)
{
    int i;

    for (i = 0; i < MAX_CALLERS; i++)
        if (g.callers[i].ptype == MAIN)
            (*g.callers[i].c.main) ();
}

void initng_close_all_fd(void)
{
    int i;

    for (i = 0; i < MAX_CALLERS; i++)
    {
        if (g.callers[i].ptype == FDWATCHERS && (g.callers[i].c.fdh.fds > 0))
        {
            close(g.callers[i].c.fdh.fds);
            g.callers[i].c.fdh.fds = 0;
            g.callers[i].c.fdh.call_module = NULL;
            g.callers[i].ptype = NOPLUG;

        }
    }

}

/* Calls plugins when data is ready in pipe */
void initng_plugin_readpipe(active_h * service, int datalen,
                            process_h * process, int force_flush)
{
    int i;

    for (i = 0; i < MAX_CALLERS; i++)
    {
        if (g.callers[i].ptype == PIPEWATCHERS)
        {
            (*g.callers[i].c.pipewatcher) (service, datalen, process,
                                           force_flush);
        }
    }
    return;
}

/* This does the actual read from pipe, all checks have been done here */
int initng_process_readpipe_read(active_h * service, process_h * x,
                                 int program_end)
{
    int forceflush = program_end;
    int maxread = MAX_BUFFER - x->buffer_pos;
    int len;

    do
    {
        errno = 0;
        len = read(x->stdout[0], &x->buffer[x->buffer_pos], maxread);
    }
    while (errno == EINTR);

    if (len > 0)
    {
        x->buffer[x->buffer_pos + len] = '\0';

        if (x->buffer_pos + len >= MAX_BUFFER)
            forceflush = TRUE;

        initng_plugin_readpipe(service, len, x, forceflush);
        x->buffer_pos = x->buffer_pos + len;
        if (forceflush)
            x->buffer_pos = 0;
    }
    return len;
}

/* This does all the fancy pancy around the read */
void initng_process_readpipe(active_h * service, process_h * x,
                             int program_end)
{
    int i;

    /* We've already quit */
    if ((!x->buffer))
        return;


    if (program_end)
    {
        int fd_flags;

        fd_flags = fcntl(x->stdout[0], F_GETFL, 0);
        fcntl(x->stdout[0], F_SETFL, fd_flags | O_NONBLOCK);
    }

    i = initng_process_readpipe_read(service, x, program_end);

    if (!program_end)
    {
        /* We have an error other than EAGAIN, set program_end */
        if ((i == -1) && (errno != EAGAIN))
        {
            program_end = TRUE;
            F_("Error reading from pipe for %s (%s)\n", service->name,
               strerror(errno));
        }
        /* End of file */
        if (i == 0)
            program_end = TRUE;
    }

    /* If i < 1 we either have 0 (end of file) or -1 (error) */
    if (program_end)
    {
        /* Program is ending, close pipes */
        initng_plugin_readpipe(service, 0, x, TRUE);
        D_("Freeing buffer for %s\n", service->name);
        if (x->stdout[0] > 0)
            close(x->stdout[0]);
        if (x->stdout[1] > 0)
            close(x->stdout[1]);
        if (x->buffer)
            free(x->buffer);
        x->stdout[0] = -1;
        x->stdout[1] = -1;
        x->buffer = NULL;
    }
}

/* this will poll all filedescriptors in g.fds[] db, for input, 
   and call the right function, when events occur 
   returns FALSE if select() timed out. */
int initng_plugin_fd_poll(int timeout)
{
    fd_set readset;
    struct timeval tv;
    int i;
    int retval;
    int set = FALSE;
    active_h *tmp = NULL;

    FD_ZERO(&readset);

    /* First scan through active plugins that have listening file descriptors, and add them */
    for (i = 0; i < MAX_CALLERS; i++)
    {
        if (g.callers[i].ptype == FDWATCHERS
            && (g.callers[i].c.fdh.fds > 0
                || g.callers[i].c.fdh.call_module != NULL))
        {
            FD_SET(g.callers[i].c.fdh.fds, &readset);
            set = TRUE;
            D_("adding fd #%i to polls found on caller %i\n",
               g.callers[i].c.fdh.fds, i);
        }
    }

    /* Then add all file descriptors from process read pipes */
    while ((tmp = active_db_next(tmp)))
    {
        if ((tmp->start_process) && (tmp->start_process->stdout[0] > 0))
        {
            FD_SET(tmp->start_process->stdout[0], &readset);
            set = TRUE;
        }
        if ((tmp->stop_process) && (tmp->stop_process->stdout[0] > 0))
        {
            FD_SET(tmp->stop_process->stdout[0], &readset);
            set = TRUE;
        }
    }

    /* check if there are any set */
    if (!set)
    {
        D_("No filedescriptors\n");
        sleep(timeout);
        return (TRUE);
    }

    tv.tv_sec = timeout;
    tv.tv_usec = 0;

    /* If interrupt then return and do main loop, it doesn't hurt if the filebuffers have a bit more data anyway */
    if (g.interrupt)
        return (TRUE);

    retval = select(256, &readset, NULL, NULL, &tv);
    if (g.interrupt)
        return (TRUE);
    if (retval == 0)
        return (FALSE);
    if (retval == -1)
        F_("Select returned with %d %s\n", retval, strerror(errno));

    D_("select retval=%i\n", retval);

    /* find matches, for calling right funtion to 
       fetch the incoming data */
    if (retval > 0)
    {
        D_("%d fd's active\n", retval);
        /* If a fsck is running select will always return one filehandler active, we give it 0.1 seconds to get some more data into the buffer. */
        sleep(0.1);
        /* Scan through fdwatchers, check for matching fd's */
        for (i = 0; i < MAX_CALLERS; i++)
            if ((g.callers[i].ptype == FDWATCHERS
                 && (g.callers[i].c.fdh.fds > 0
                     || g.callers[i].c.fdh.call_module != NULL))
                && (FD_ISSET(g.callers[i].c.fdh.fds, &readset)))
            {
                D_("Calling pluginhandler for fd #%i\n",
                   g.callers[i].c.fdh.fds);
                (*g.callers[i].c.fdh.call_module) ();
                D_("pluginhandler for fd #%i returned!\n",
                   g.callers[i].c.fdh.fds);

                /* Found match, that means we need to look for one less, if we've found all we should then return */
                retval--;
                if (retval == 0)
                    return (TRUE);
            }
        /* Now scan through processes */
        tmp = NULL;
        while ((tmp = active_db_next(tmp)))
        {
            if ((tmp->start_process)
                && (tmp->start_process->stdout[0] > 0)
                && (FD_ISSET(tmp->start_process->stdout[0], &readset)))
            {
                /* Do the actual read from pipe */
                initng_process_readpipe(tmp, tmp->start_process, FALSE);
                /* If we've been interrupted, return to main loop (program might have ended) */
                if (g.interrupt)
                    return (TRUE);
                /* Found match, that means we need to look for one less, if we've found all we should then return */
                retval--;
                if (retval == 0)
                    return (TRUE);
            }
            if ((tmp->stop_process)
                && (tmp->stop_process->stdout[0] > 0)
                && (FD_ISSET(tmp->stop_process->stdout[0], &readset)))
            {
                /* See above */
                initng_process_readpipe(tmp, tmp->stop_process, FALSE);
                if (g.interrupt)
                    return (TRUE);
                retval--;
                if (retval == 0)
                    return (TRUE);
            }
        }
    }
    return (TRUE);
}

service_h *initng_modules_parse(char *name)
{
    int i;
    service_h *to_ret;

    for (i = 0; i < MAX_CALLERS; i++)
    {
        if (g.callers[i].ptype == PARSERS && g.callers[i].c.parser)
        {
            to_ret = (*g.callers[i].c.parser) (name);
            if (to_ret)
                return (to_ret);
        }
    }
    return (NULL);
}

/* called when a service status has changed. */
void initng_load_module_service_changed(active_h * service)
{
    int i;

    for (i = 0; i < MAX_CALLERS; i++)
        if (g.callers[i].ptype == WATCHERS && g.callers[i].c.watcher)
            (*g.callers[i].c.watcher) (service);
    return;
}

/* called when system state has changed. */
void initng_load_module_system_changed(h_sys_state state)
{
    int i;

    for (i = 0; i < MAX_CALLERS; i++)
        if (g.callers[i].ptype == SWATCHERS && g.callers[i].c.swatcher)
            (*g.callers[i].c.swatcher) (state);
    return;
}
