
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "initng.h"

#define _GNU_SOURCE
#include <stdlib.h>                         /* malloc() free() exit() */
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdarg.h>                         /* vastart() vaend() */

#include "initng_toolbox.h"
#include "initng_global.h"

/* in init, a calloc must never fail! */
void *initng_calloc2(size_t nmemb, size_t size, const char *func, int line)
{
    void *alloced = NULL;

    D_("function %s() line %i allocating %i bytes!\n", func, line,
       nmemb * size);
    while ((alloced = calloc(nmemb, size)) == NULL)
    {
        F_("function %s() line %i : ( %i bytes) Out of memory, allocation problem!", func, line, nmemb * size);
        sleep(1);                           /* sleep one second */
    }
    return (alloced);
}

/* in init, a realloc must never fail! */
void *initng_realloc(void *ptr, size_t size)
{
    void *alloced = NULL;

    while ((alloced = realloc(ptr, size)) == NULL)
    {
        F_("initng_realloc(): Out of memory, allocation problem!");
        usleep(100000);                     /* sleep one second */
    }
    return (alloced);
}

/* in init, a strndup must never fail! */
char *i_strndup(const char *s, size_t n)
{
    char *alloced = NULL;

    while ((alloced = strndup(s, n)) == NULL)
    {
        F_("i_strndup: Out of memory, allocation problem!");
        sleep(1);                           /* sleep one second */
    }
    return (alloced);
}

/* in init, a strdup must never fail! */
char *i_strdup(const char *s)
{
    char *alloced = NULL;

    while ((alloced = strdup(s)) == NULL)
    {
        F_("i_strdup: Out of memory, allocation problem!");
        sleep(1);
    }
    return (alloced);
}


/* stolen from sysvinit */
int set_proc_title(const char *fmt, ...)
{
    va_list ap;
    int len;
    char *buf;

    buf = initng_calloc(1, g.maxproclen + 1);
    buf[0] = '\0';

    va_start(ap, fmt);
    len = vsnprintf(buf, (g.maxproclen * sizeof(char)), fmt, ap);
    va_end(ap);
    /* we cant realloc here, so last string, have to be as large as this string. */
    if (g.maxproclen > len)
    {
        memset(g.Argv0, 0, g.maxproclen);   /* clear */
        strcpy(g.Argv0, buf);               /* copy */
        D_("g.Argv0: %s\n", g.Argv0);
    }
    else
    {
        len = 0;
        D_("Cant reset own argv[0].\n");
    }

    free(buf);
    return len;
}
