
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "initng.h"

#define _GNU_SOURCE
#include <fnmatch.h>

#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include "string_tools.h"
#include "initng_toolbox.h"

char **split_n(char *buf, int *len, int plus, int ofs)
{
    int n = 1;
    char **v = 0;
    char **w;

    /* step 1: count tokens */
    char *s;

    for (s = buf; *s; s++)
        if (*s == '\n' || *s == ',' || *s == ';' || *s == ' ')
            n++;
    /* step 2: allocate space for pointers */
    v = (char **) initng_calloc((n + plus), sizeof(char *));
    if (!v)
        return 0;
    w = v + ofs;
    *w++ = buf;
    for (s = buf;; s++)
    {
        while (*s && *s != '\n' && *s != ',' && *s != ';' && *s != ' ')
            s++;
        if (*s == 0)
            break;
        if (*s == '\n' || *s == ',' || *s == ';' || *s == ' ')
        {
            *s = 0;
            *w++ = s + 1;
        }
    }
    *len = w - v;
    return v;
}

int chars_to_end_of_this_stack(char *string)
{
    int i = 0;
    int stacks = 1;

    while (string[i] != '\0')
    {
        if (string[i] == '{')
            stacks++;
        if (string[i] == '}')
        {
            stacks--;
            if (stacks == 0)
                return (i);
        }
        i++;
    }
    return (-1);
}

/* return word length */

/*
   static int st_wl(char *string){
   int i=0;
   if(string[0]=='"') {
   string++;
   while(string[i]!='\0' &&
   string[i]!='"'  &&
   string[i]!='\n' &&
   string[i]!=','  &&
   string[i]!=';') i++;
   }      
   else
   while(string[i]!='\0' &&
   string[i]!=' '  &&
   string[i]!='\t' &&
   string[i]!='\n' &&
   string[i]!=','  &&
   string[i]!=';') i++;

   return(i);
   } */

int st_cmp(char **string, const char *to_cmp)
{
    int chars = strlen(to_cmp);

    if (strncasecmp((*string), to_cmp, chars) == 0)
    {
        /* increase the string pointer */
        (*string) = (*string) + chars;
        return (TRUE);
    }
    return (FALSE);
}

char *st_dup_next_word(char **string)
{
    char *td = NULL;
    int i = 0;

    /* count tokens */
    if (*(string)[0] == '"')
    {
        (*string)++;
        while ((*string)[i] != '\0' &&
               (*string)[i] != '"' &&
               (*string)[i] != '\n' &&
               (*string)[i] != ',' && (*string)[i] != ';')
            i++;
    }
    else
        while ((*string)[i] != '\0' &&
               (*string)[i] != ' ' &&
               (*string)[i] != '\t' &&
               (*string)[i] != '\n' && (*string)[i] != ','
               && (*string)[i] != ';')
            i++;

    if (!i)
        return (NULL);

    /* copy string */
    td = i_strndup(*string, i);
    (*string) = (*string) + i;
    if ((*string)[0] == '"')
        (*string)++;
    return (td);
}

char *st_dup_line(char **string)
{
    char *td = NULL;
    int i = 0;

    /* count tokens */
    if ((*string)[0] == '"')
    {
        (*string)++;
        while ((*string)[i] != '\0' &&
               (*string)[i] != '"' &&
               (*string)[i] != '\n' && (*string)[i] != ';')
            i++;
    }
    else
        while ((*string)[i] != '\0' &&
               (*string)[i] != '\n' && (*string)[i] != ';')
            i++;

    if (!i)
        return (NULL);

    /* copy string */
    td = i_strndup(*string, i);
    (*string) = (*string) + i;

    /* fast forward */
    while ((*string)[0] != '\0' && (*string)[0] != '\n'
           && (*string)[0] != ';')
        (*string)++;
    return (td);
}

void st_jump_spaces(char **string)
{
    while ((*string)[0] == ' ' || (*string)[0] == '\t' || (*string)[0] == '=')
        (*string)++;
}

void st_jump_to_next_row(char **string)
{
    /* on end, step to end of row */
    while ((*string)[0] != '\0' && (*string)[0] != '\n'
           && (*string)[0] != ';')
        (*string)++;
    /* and take a step to beginning of next row */
    while ((*string)[0] == '\n' || (*string)[0] == ';')
        (*string)++;
    /* go to a valid line */
    st_jump_to_valid_line(string);
}

void st_jump_to_valid_line(char **string)
{
    /* jump all the spaces */
    while ((*string)[0] == ' ' || (*string)[0] == '\t' || (*string)[0] == '=')
        (*string)++;
    /* all rows starting with '#','\n',';' are disabled */
    while ((*string)[0] == '#')
    {
        /* jump straight to next row, no ';' pass here */
        while ((*string)[0] != '\0' && (*string)[0] == '\n')
            (*string)++;
        st_jump_to_next_row(string);
    }
    while ((*string)[0] == '\n' || (*string)[0] == ';')
        st_jump_to_next_row(string);
}

/* function strips "test/go/th" to "th" */
char *st_strip_path(char *string)
{
    int i = 0;

    /* go to end */
    while (string[i] != '\0')
        i++;

    /* go back to the '/' char */
    while (i && string[i] != '/')
        i--;

    if (string[i] == '/')
        return (&string[i + 1]);
    return (string);
}

/* function strips "test/go/th" to "test/go" */
char *st_get_path(char *string)
{
    int i = 0;

    /* go to end */
    while (string[i] != '\0')
        i++;
    /* step back */
    while (i && string[i] != '/')
        i--;

    if (i)
        return (i_strndup(string, i));

    return (i_strdup(string));
}

/* function strips "test/go/th" to "test/go" and then to "test" */
int st_strip_end(char **string)
{
    int i = 0;

    /* stand on last char */
    while ((*string)[i] != '\0')
        i++;

    /* rewind util standing on a '/' */
    while (i && (*string)[i] != '/')
        i--;

    /* if we really are standing on a '/' */
    if ((*string)[i] == '/')
    {
        (*string)[i] = '\0';                /* truncate */
        return (TRUE);
    }
    return (FALSE);
}

/* "test/go/net/eth*" should be true to "test/go/net/eth1"
   or "test/go/net/ethernet", s1 have to be right,
   s2 can include * at end. */
int st_service_cmp(char *s1, char *s2)
{
    if (fnmatch(s2, s1, FNM_CASEFOLD) == 0)
        return (TRUE);
    return (FALSE);
}
