
/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "initng.h"
#include "initng_global.h"

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include "struct_data.h"
#include "static_data_id.h"
#include "initng_toolbox.h"

char *d_get_string(s_entry * type, s_data * d)
{
    assert(d);

    /* check so that data entry exists */
    if (!type)
        return (NULL);

    /* if this type is right, return string */
    if (d->type == type && d->t.s)
    {
        D_("got %s\n", d->t.s);
        return (d->t.s);
    }

    /* else, check if next will return something */
    if (d->next)
        return (d_get_string(type, d->next));

    /* did not get any match */
    D_("String type %s doesn't exist.\n", type->opt_name);
    return (NULL);
}

int d_get_int(s_entry * type, s_data * d)
{
    assert(d);

    if (!type)
        return (0);


    /* if this type is right, return int */
    if (d->type == type && d->t.i)
        return (d->t.i);


    /* else, check if next will return something */
    if (d->next)
        return (d_get_int(type, d->next));

    /* did not get any match */
    D_("Int value, type %s doesn't exist in db.\n", type->opt_name);
    return (0);
}

char **d_get_stringlist(s_entry * type, s_data * d)
{
    assert(d);

    if (!type)
        return (0);

    /* if this type is right, return string list */
    if (d->type == type && d->t.l)
        return (d->t.l);

    /* else, check if next will return something */
    if (d->next)
        return (d_get_stringlist(type, d->next));

    /* did not get any match */
    D_("Stringlist value, type %s doesn't exist in db.\n", type->opt_name);
    return (0);
}

char *d_get_next_string(s_entry * type, s_data * d, char *last)
{

    assert(d);
    if (!type)
        return (NULL);

    /* if no last, return first */
    if (last == NULL)
        return (d_get_string(type, d));

    /* if this is last, return next string */
    if (d->type == type && d->t.s == last && d->next)
    {
        return (d_get_string(type, d->next));
    }


    /* if next, try it too */
    if (d->next)
        return (d_get_next_string(type, d->next, last));

    /* no match */
    return (NULL);
}

/* this has to be only one of */
void d_set_string(s_entry * type, s_data * d, char *string)
{
    assert(d);
    assert(string);
    if (!type)
        return;


    /* check if this is a free spot, or an old of this type */
    if (d->type == NULL || d->type == type)
    {
        d->type = type;
        if (d->t.s)
            free(d->t.s);
        d->t.s = string;
        return;
    }

    /* else, go forward, looking for a free spot */
    if (!d->next)
    {
        d->next = (s_data *) initng_calloc(1, sizeof(s_data));
    }
    d_set_string(type, d->next, string);
    return;
}

#ifdef ENABLE_UNTESTED
/* this add a string to a string list */
int d_stringlist_add(s_entry * type, s_data * d, char *string)
{
    char **sl = NULL;
    int i = 0;

    /* check if a stringlist exists */
    sl = d_get_stringlist(type, d);

    /* else, allocate and add. */
    if (!sl)
    {
        sl = initng_calloc(20, sizeof(char *));
        d_set_stringlist(type, d, sl);
    }

    for (i = 0; i < 20; i++)
    {
        if (sl[i] == NULL)
            sl[i] == string;
        return (TRUE);
    }
    return (FALSE);
}

int d_stringlist_del(s_entry * type, s_data * d, char *string)
{
    char **sl = NULL;
    int i = 0;

    /* check if a stringlist exists */
    if (!(sl = d_get_stringlist(type, d)))
        return (FALSE);

    for (i = 0; i < 20; i++)
    {
        if (strcmp(string, sl[i]) == 0)
        {
            free(sl[i]);
            sl[i] = NULL;
            return (TRUE);
        }
    }
    return (FALSE);
}
#endif

void d_set_stringlist(s_entry * type, s_data * d, char **string)
{
    assert(d);
    assert(string);
    if (!type)
        return;


    /* check if this is a free spot, or an old of this type */
    if (d->type == NULL || d->type == type)
    {
        d->type = type;
        if (d->t.l)
            free(d->t.l);
        d->t.l = string;
        return;
    }

    /* else, go forward, looking for a free spot */
    if (!d->next)
    {
        d->next = (s_data *) initng_calloc(1, sizeof(s_data));
    }
    d_set_stringlist(type, d->next, string);
    return;
}

/* this is a function to add a string, that it can exist many of */
void d_set_another_string(s_entry * type, s_data * d, char *string)
{
    assert(d);
    assert(string);
    if (!type)
        return;

    /* check if this is a free spot */
    if (!d->type)
    {
        d->type = type;
        d->t.s = string;
        return;
    }

    /* else, go forward, looking for a free spot */
    if (!d->next)
    {
        d->next = (s_data *) initng_calloc(1, sizeof(s_data));
    }
    d_set_another_string(type, d->next, string);
    return;
}

/* this has to be only one of */
void d_set_int(s_entry * type, s_data * d, int value)
{
    assert(d);
    if (!type)
        return;

    if (type->opt_type != INT)
    {
        F_(" \"%s\" is not an int type!\n", type->opt_name);
        return;
    }

    /* check if this is a free spot, or an old of this type */
    if (d->type == NULL || d->type == type)
    {
        d->type = type;
        d->t.i = value;
        return;
    }

    /* else, go forward, looking for a free spot */
    if (!d->next)
    {
        d->next = (s_data *) initng_calloc(1, sizeof(s_data));
    }
    d_set_int(type, d->next, value);
    return;
}



/* Walk through and count */
int d_count_type(s_entry * type, s_data * d)
{
    /* check that this service exists or return zero */
    if (!d || !type)
        return (0);

    /* If this is what we are looking for, return next also */
    if (d->type == type)
        return (d_count_type(type, d->next) + 1);

    return (d_count_type(type, d->next));
}

int d_is(s_entry * type, s_data * d)
{
    assert(d);
    if (!type)
    {
        F_("d_is with type %d\n", type);
        return (FALSE);
    }


    if (d->type == type)
        return (TRUE);

    if (d->next)
        return (d_is(type, d->next));

    return (FALSE);
}

void d_set(s_entry * type, s_data * d)
{
    assert(d);
    if (!type)
        return;


    /* check if this is a free spot, or an old of this type */
    if (d->type == NULL || d->type == type)
    {
        d->type = type;
        return;
    }

    /* else, go forward, looking for a free spot */
    if (!d->next)
    {
        d->next = (s_data *) initng_calloc(1, sizeof(s_data));
    }
    d_set(type, d->next);
    return;
}

void d_remove(s_entry * type, s_data * d)
{
    assert(d);
    assert(type);

    if (!type)
    {
        F_("type not set!\n");
        return;
    }

    if (d->next)
        d_remove(type, d->next);

    if (d->type == type)
    {
        if ((d->type->opt_type == STRING || d->type->opt_type == STRINGS
             || d->type->opt_type == SCRIPT) && d->t.s)
        {
            D_("Freeing string \"%s\"\n", d->t.s);
            free(d->t.s);
        }
        d->t.s = NULL;
        d->type = NULL;
    }
}

/* walk through and kill all strings */
void d_remove_all(s_data * d)
{
    assert(d);
    if (d->next)
    {
        d_remove_all(d->next);
        d->next = NULL;
    }

    if (d->type
        && (d->type->opt_type == STRING || d->type->opt_type == STRINGS
            || d->type->opt_type == SCRIPT) && d->t.s)
    {
        D_("Freeing string \"%s\"\n", d->t.s);
        free(d->t.s);
    }
    d->t.s = NULL;
    d->type = NULL;
    free(d);
    return;
}
