/* Initng, a next generation sysvinit replacement.
 * Copyright (C) 2005 Jimmy Wennlund <jimmy.wennlund@gmail.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#define TRUE 1
#define FALSE 0
#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>


/* this program will parse all .i files in installation
   setting distribution dependent values on install time */

static int is_file(const char *filename)
{
    struct stat tmp;

    if (stat(filename, &tmp) == 0 && S_ISREG(tmp.st_mode))
        return (TRUE);
    return (FALSE);
}

static const char *probe_distribution(void)
{
    if (is_file("/etc/debian_version"))
        return ("debian");
    if (is_file("/etc/gentoo-release"))
        return ("gentoo");
    return (NULL);
}


int main(int argc, char **argv)
{
    int i = 1;
    char *in_buf = NULL;
    char *out_buf = NULL;
    char *pt = NULL;
    off_t fsize = 0;
    int f_in;
    int f_out;
    int print_row = 1;
    const char *distro = NULL;
    int distro_len;
    const char *in = NULL;
    const char *out = NULL;

    (void) argc;

    /* check cmdline for set distribution */
    while (argv[i])
    {

        if (argv[i + 1])
        {
            if (strcmp(argv[i], "--dist") == 0 || strcmp(argv[i], "-d") == 0)
            {
                distro = argv[i + 1];
                i++;
            }
            else if (strcmp(argv[i], "--in") == 0
                     || strcmp(argv[i], "-i") == 0)
            {
                in = argv[i + 1];
                i++;
            }
            else if (strcmp(argv[i], "--out") == 0
                     || strcmp(argv[i], "-o") == 0)
            {
                out = argv[i + 1];
                i++;
            }
        }
        i++;
    }

    /* if not set, probe it */
    if (!distro)
        distro = probe_distribution();
    if (!distro)
    {
        fprintf(stderr, "Unknown DISTRIBUTION\n");
        exit(1);
    }
    distro_len = strlen(distro);
    fprintf(stderr, "Distribution set: \"%s\"\n", distro);

    if (!in)
    {
        fprintf(stderr, "Have to set input file with -i\n");
        exit(1);
    }


    f_in = open(in, O_RDONLY);
    if (!f_in)
    {
        fprintf(stderr, "Unable to open input file %s\n", in);
        return (1);
    }

    /* get in size, so we not how mutch to alloc */
    fsize = lseek(f_in, 0, SEEK_END);
    lseek(f_in, 0, SEEK_SET);

    /* allocate in and out buf */
    in_buf = calloc(fsize + 1, sizeof(char));
    out_buf = calloc(fsize + 11, sizeof(char));

    /* zero outbuf */
    out_buf[0] = '\0';

    /* read input */
    read(f_in, in_buf, fsize);

    /* close input */
    close(f_in);

    /*fprintf(stderr,"File (%s): \"%s\"\n\n\n", in, in_buf); */

    /* set a temporary pointer */
    pt = in_buf;

    /* go tru every row */
    while (pt[0])
    {
        int row_len = 0;
        char *tmp = NULL;

        while (pt[row_len] != '\n' && pt[row_len])
            row_len++;
        if (pt[row_len] == '\n')
            row_len++;

        /* if we are gonna handle this output */
        if (pt[0] == '#')
        {
            if (strncmp("#ifd", pt, 4) == 0)
            {
                print_row = 0;
                tmp = strstr(pt + 4, distro);
                /* if distro was found, and on this row */
                if (tmp && (tmp - pt) < row_len)
                    print_row = 1;
                goto next;
            }
            else if (strncmp("#elsed", pt, 6) == 0)
            {

                /* check variables */
                tmp = strstr(pt + 6, distro);
                /* if distro was found, and on this row */
                if (tmp && (tmp - pt) < row_len)
                {
                    print_row = 1;
                    goto next;
                }

                /* else youst swap */
                if (print_row == 1)
                    print_row = 0;
                else
                    print_row = 1;

                goto next;
            }
            else if (strncmp("#endd", pt, 5) == 0)
            {
                /* print as usual */
                print_row = 1;
                goto next;
            }
        }

        /* else we youst forwards - calculate row lenght */
        if (print_row)
            strncat(out_buf, pt, row_len);

      next:
        /* go to end */
        while (pt[0] != '\n' && pt[0])
            pt++;
        /* go to next row */
        if (pt[0] == '\n')
            pt++;
    }

    /* free input buffer */
    free(in_buf);

    /* open output for writing */
    if (out)
    {
        f_out = open(out, O_WRONLY + O_CREAT);
        /* write output */
        write(f_out, out_buf, strlen(out_buf));
        close(f_out);
    }
    else
    {
        /* if its a .ii file */
        if (strstr(in, ".ii"))
        {
            char *out_2 = NULL;

            /* take away the last i */
            out_2 = strndup(in, strlen(in) - 1);
            f_out = open(out_2, O_WRONLY + O_CREAT);
            /* write output */
            write(f_out, out_buf, strlen(out_buf));
            close(f_out);
        }
        else
        {

            fprintf(stderr, "out with -o not set, defaulting to stdout.\n");
            printf("%s", out_buf);

        }
    }

    /* free output buffer */
    free(out_buf);

    exit(0);
}
